/***************
 * NAME:            "IconClone" - version 1.0.9
 * DESCRIPTION:     Functions for exchanging icon images
 * AUTHOR:          (c) Mike Riddle - initial release 12-14-21                     
 ***************/


/* REFERENCES 12-14-21 */

// http://krashan.ppa.pl/articles/amigaicons/ (HTML - Amiga Icon File Format)
// http://www.evillabs.net/index.php/Amiga_Icon_Formats (Text - Dirk S.)
// all Amiga icons formats are supported except oldicons/newIcons


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <dos/dos.h>
#include <exec/exec.h>

#include <proto/wb.h>
#include <proto/icon.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/asl.h>
#include <proto/icon.h>
#include <proto/graphics.h>
#include <proto/workbench.h>
#include <proto/intuition.h>
#include <proto/datatypes.h>
#include <proto/gadtools.h>
#include <proto/iffparse.h>
#include <libraries/iffparse.h>

#include <workbench/icon.h>
#include <workbench/workbench.h>
#include <workbench/startup.h>
#include <intuition/intuition.h>
#include <datatypes/datatypes.h>
#include <datatypes/pictureclass.h>
#include <datatypes/datatypesclass.h>
#include <proto/cybergraphics.h>
#include <cybergraphx/cybergraphics.h>

 
#define GID_REQ1  4714 
#define GID_REQ2  4715 
#define GID_EXCH  4716

#define MAX_IMAGE_WIDTH 80
#define MAX_IMAGE_HEIGHT 80

#define ID_ICON MAKE_ID('I','C','O','N')
#define ID_FACE MAKE_ID('F','A','C','E')
#define ID_IMAG MAKE_ID('I','M','A','G')
#define ID_ARGB MAKE_ID('A','R','G','B')

/* Values for ICONCTRLA_GetImageDataFormat */
#define ICONCTRLA_SetImageDataFormat        (ICONA_BASE + 0x67) /*103*/
#define ICONCTRLA_GetImageDataFormat        (ICONA_BASE + 0x68) /*104*/

/* Values for ICONCTRLA_Set/GetImageDataFormat */
#define IDFMT_BITMAPPED     (0)  /* Bitmapped icon (planar, legacy) */
#define IDFMT_PALETTEMAPPED (1)  /* Palette mapped icon (chunky, V44+) */
#define IDFMT_DIRECTMAPPED  (2)  /* Direct mapped icon (truecolor 0xAARRGGBB, V51+) */

/* Defines for Get Data Functions - based on modifyicon */
#define GetUint32(a)  (((((unsigned char *)a)[0])<<24)|((((unsigned char *)a)[1])<<16)| \
                       ((((unsigned char *)a)[2])<< 8)|((((unsigned char *)a)[3])))
#define GetUint16(a)  (((((unsigned char *)a)[0])<< 8)|((((unsigned char *)a)[1]))) //scrwidth = filebuf[6] | filebuf[7] << 8;

//#define GetUint16BE(a)  (((((unsigned char *)a)[0]))|((((unsigned char *)a)[1])<< 8))
//#define GetUint16LE(a)  (((((unsigned char *)a)[0])>> 8)|((((unsigned char *)a)[1])))

#define RGB8to32(RGB) (((ULONG)(RGB) << 24)|((ULONG)(RGB) << 16)|((ULONG)(RGB) << 8)|((ULONG)(RGB)))


WORD topoffset;
WORD leftoffset;
WORD deltasize;
WORD windowsize;
WORD buttonoffset;

STRPTR filename = NULL;
STRPTR srcename = NULL;
STRPTR destname = NULL; 
STRPTR tempname = NULL;
STRPTR iconfile = NULL;
STRPTR srcefile = NULL;
STRPTR destfile = NULL;
STRPTR tempfile = NULL;
STRPTR srcepath = NULL;

char *srceicontype = NULL;
char *desticontype = NULL;


/* The List of Active Gadgets */
struct Gadget *glist = NULL, *butgad1, *butgad2, *butgad3;

/* Setup File Requesters */
struct FileRequester *SrcFileReq  = NULL, *DestFileReq = NULL;

BOOL retval = FALSE;
UBYTE filenamebuffer[300];

/* Setup Disk Objects */
struct DiskObject *srcedobj = NULL;
struct DiskObject *destdobj = NULL;


/****************************************************************************************/

struct FileFaceChunk
{
    UBYTE Width;
    UBYTE Height;
    UBYTE Flags;
    UBYTE Aspect;
    UBYTE MaxPaletteBytes[2];
};

struct FileImageChunk
{
    UBYTE TransparentColor;
    UBYTE NumColors;
    UBYTE Flags;
    UBYTE ImageFormat;
    UBYTE PaletteFormat;
    UBYTE Depth;  
    UBYTE NumImageBytes[2];
    UBYTE NumPaletteBytes[2];
};

struct IconInfo 
{
	ULONG Width;
	ULONG  Height;
	ULONG  Format;
	ULONG  NumColors;
	const struct ColorRegister *CMap;
	UBYTE *Image;
	ULONG  Which;
	ULONG  NumImages;
};

/****************************************************************************************/

BOOL EXISTS(char *fname);
char *get_ext(char* path);
char *GetPath(char *path);
LONG GetFileSize(char *imName);
LONG Get_IFFOffset(char *srcename);

char *RemoveFileExtension(char *path);
char* GetFileNameFromPath(char* path);
void SetFilesize(char *name, LONG newsize);
char *Check_Icon_Signature(char *srcename);
struct BitMapHeader *readbmhd(char *file_name);
char *Check_IFFType(char *srcename, LONG iffoffset);
const char *DetectBaseName_by_datatype(char *file_name);
void OutputMessage(STRPTR outputmsg, struct Window *win);
void Make_Icon(char *image1, char *image2, char *outfile);
BOOL inRectangle(int x,int y, int w, int h,int tx, int ty);

void Copy_Icon(char *destname);
void Copy_Data(char *srcename, char *destname, LONG filebytes); //Should only copy data
void Copy_File(char *srcename, char *destname, LONG filebytes);
STRPTR GetSourceFilename(char * asl_title, struct Window *win);
STRPTR GetDestinationFilename(char * asl_title, struct Window *win);
BOOL ReadPixels_by_datatype(UBYTE *pixels, char *file_name, int bpp);
BOOL ReadImage (UBYTE *pixels, struct IconInfo info, int bpp);
BOOL SavePixels_by_datatype(ULONG *data, char *filename, int width, int height);
BOOL Save_Icon(struct DiskObject *oldicon, struct DiskObject *newicon, STRPTR name);
void Copy_File_Append(char *srcename, char *destname, LONG offset, LONG filebytes);
//BOOL Display_Icon(char *srcename, struct DiskObject *icon, char *position);
BOOL Display_Icon(char *srcename, struct DiskObject *icon, char *position, WORD deltasize, struct Window *win);
void Display_Rectangle(char *color, char *position, WORD deltasize, struct Window *win);
void Exchange_Icon(struct DiskObject *dobj1, struct DiskObject *dobj2, char *destname, WORD deltasize, struct Window *win);
static LONG MyMemStreamHandler(struct Hook *hook, struct IFFHandle * iff, struct IFFStreamCmd * cmd);


/*********************************************************************************************/

BOOL EXISTS(char *fname)
{
    BPTR file;    
    file = Open(fname, MODE_OLDFILE);
	if (!file) 
	{
		Close(file);
        return TRUE;
	}
    return FALSE;
}

BOOL inRectangle(int x,int y, int w, int h,int tx, int ty)
{
  if (tx >=x  && tx <= x+w && ty >=y && ty <= y+h) return(TRUE); else return(FALSE);
}

static LONG MyMemStreamHandler(struct Hook *hook, struct IFFHandle * iff, struct IFFStreamCmd * cmd)
{
    LONG error = 0;
    APTR *buffp = (APTR *)iff->iff_Stream;
    APTR buff = *buffp;

    switch(cmd->sc_Command)
    {
    	case IFFCMD_INIT:
	case IFFCMD_CLEANUP:
	    error = 0;
	    break;
	    
	case IFFCMD_READ:
	    error = 0;
	    CopyMem(buff, cmd->sc_Buf, cmd->sc_NBytes);
	    (*buffp) += cmd->sc_NBytes;
	    break;
	    
	case IFFCMD_WRITE:
	    error = 0;
	    CopyMem(cmd->sc_Buf, buff, cmd->sc_NBytes);
	    (*buffp) += cmd->sc_NBytes;
	    break;
	    
	case IFFCMD_SEEK:
	    (*buffp) += cmd->sc_NBytes;
	    error = 0;
	    break;
    }
    
    return error;
}

/*********************************************************************************************/

void OutputMessage(STRPTR msg, struct Window *win)
{
    struct EasyStruct es;    
   
    if (msg)
    {        
        es.es_StructSize   = sizeof(es);
        es.es_Flags        = 0;
        es.es_Title        = "Message";
        es.es_TextFormat   = msg;
        es.es_GadgetFormat = "OK";
           
        EasyRequestArgs(win, &es, NULL, NULL);        
    }
}

/*********************************************************************************************/

LONG Get_IFFOffset(char *srcename)
{	
	LONG offset;
	LONG i, j, k;	
	UWORD FileID;	
	UBYTE Buffer16[2];
	UBYTE Buffer32[4];	
	WORD width, height, depth;
	int imagesize, imageoffset;

	//Printf("\nGet_IFFOffset...\n");

	LONG firstimage, secondimage;	
	LONG tooltypes, deftool, drawerdata, toolwin;

	/* Open icon file to Read */
    BPTR fh = Open(srcename, MODE_OLDFILE);
    if (fh)
	{
		offset = 0;        		  

        Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer16, 2);	    		
		if( Buffer16[0] == 0xE3 && Buffer16[1] == 0x10 ) 
		//Printf ("Found Classic Icon File!\n");

		offset += 2;
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer16, 2);		
		FileID = GetUint16(Buffer16);
		if(FileID == 1)
		//Printf("Found Valid Version Number!\n");

		offset += 20; //22
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);
		//secondimage = EndGetM32(buffer+0x16) ? 1 : 0;		
		//firstimage = GetUint32(Buffer32) ? 1: 0; //22
		firstimage = (GetUint32(Buffer32)==0) ? 0: 1; //22
		//Printf ("First Image: %ld, %ld\n", firstimage, offset);

		offset += 4; //26
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);
		//secondimage = GetUint32(Buffer32) ? 1: 0; //26
		secondimage = (GetUint32(Buffer32)==0) ? 0: 1; //26		
		//Printf ("Second Image: %ld, %ld\n", secondimage, offset);

		offset += 24; //26+24=50
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);
		//deftool = GetUint32(Buffer32) ? 1: 0; //50
		deftool = (GetUint32(Buffer32)==0) ? 0: 1; //50
		//Printf ("DiskObject - deftool: %ld, %ld\n", deftool, offset);
		offset += 4; //54
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);
		//tooltypes = GetUint32(Buffer32) ? 1: 0; //54
		tooltypes = (GetUint32(Buffer32)==0) ? 0: 1; //54
        //Printf ("DiskObject - tooltypes: %ld, %ld\n", tooltypes, offset);
		offset += 12; //66
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);
        //drawerdata = GetUint32(Buffer32) ? 1: 0; //66
		drawerdata = (GetUint32(Buffer32)==0) ? 0: 1; //66
		//Printf ("DiskObject - drawerdata: %ld, %ld\n", drawerdata, offset);        
		offset += 4; //70
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);
		//toolwin = GetUint32(Buffer32) ? 1: 0; //70
		toolwin = (GetUint32(Buffer32)==0) ? 0: 1; //70
		//Printf ("DiskObject - toolwin: %ld, %ld\n", toolwin, offset);
		offset += 8; //64+14=78
		//Printf("Current Offset: %ld\n", offset); //78 ??

		if(drawerdata == 1)
        {		    
		    offset += 56; //78+56=134
		}
		if(firstimage == 1)
		{
			offset += 4;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
            width = GetUint16(Buffer16); //offset + 4			           
			//Printf ("First Image Width: %ld, 0x%x%x, %ld\n", width, Buffer16[0],Buffer16[1], offset);

			offset += 2;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
			height = GetUint16(Buffer16); //offset + 6			
			//Printf ("First Image Height: %ld, 0x%x%x, %ld\n", height, Buffer16[0],Buffer16[1], offset);

			offset += 2;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
			depth = GetUint16(Buffer16); //offset + 8			
			//Printf ("First Image Depth: %ld, 0x%x%x, %ld\n", depth, Buffer16[0],Buffer16[1], offset);

			offset += 12; //offset + 20
			imagesize = (((width+15)&(~15)) * height);
			//imageoffset = (((((width+15)&(~15)) * height) * depth)>>3);
			imageoffset = ((imagesize * depth)>>3);

			//Printf ("First Image Size: %ld, %ld\n", imageoffset, offset);
			offset += imageoffset;
		}
		if(secondimage == 1)
		{
			offset += 4;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
            width = GetUint16(Buffer16); //offset + 4			           
			//Printf ("Second Image Width: %ld, 0x%x%x, %ld\n", width, Buffer16[0],Buffer16[1], offset);

			offset += 2;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
			height = GetUint16(Buffer16); //offset + 6			
			//Printf ("Second Image Height: %ld, 0x%x%x, %ld\n", height, Buffer16[0],Buffer16[1], offset);

			offset += 2;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
			depth = GetUint16(Buffer16); //offset + 8			
			//Printf ("Second Image Depth: %ld, 0x%x%x, %ld\n", depth, Buffer16[0],Buffer16[1], offset);
			
			offset += 12; //offset + 20
			imagesize = (((width+15)&(~15)) * height);
			//imageoffset = (((((width+15)&(~15)) * height) * depth)>>3);
			imageoffset = ((imagesize * depth)>>3);						

			//Printf ("Second Image Size: %ld, %ld\n", imageoffset, offset);
			offset += imageoffset;
		}
		if(deftool == 1)
        {
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer32, 4);
			i = GetUint32(Buffer32);            
			offset += i+4;
			//Printf("Current Offset: %ld\n", offset);
		}
		if(tooltypes == 1)
		{
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer32, 4);
			i = GetUint32(Buffer32)/4-1;
			offset += 4;
			
			for(j = 0; j < i; ++j)
            {
				Seek(fh,offset,OFFSET_BEGINNING);
		        Read(fh,Buffer32, 4);
                k = GetUint32(Buffer32);
				offset += k+4;
            }
			//Printf("Current Offset: %ld\n", offset);
		}
		if(toolwin == 1)
        {
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer32, 4);
            i = GetUint32(Buffer32);        
		    offset += i+4; 
			//Printf("Current Offset: %ld\n", offset);
        }
		/* Check for Beginning of OS3.5 IFF Data */
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4); //0x464F524D) = 'FORM'		
		if( Buffer32[0] == 0x46 && Buffer32[1] == 0x4F && Buffer32[2] == 0x52 && Buffer32[3] == 0x4D)
		{		
		    //Printf("Found Beginning of OS3.5 IFF Data: %ld\n", offset);
			return offset;
		}
		else
		{
			/* Check Again! */
			offset += 6; //Skip 6 zero bytes at end
			Seek(fh,offset,OFFSET_BEGINNING);
			Read(fh,Buffer32, 4);
			if( Buffer32[0] == 0x46 && Buffer32[1] == 0x4F && Buffer32[2] == 0x52 && Buffer32[3] == 0x4D)
		    {		
		        //Printf("Found Beginning of OS3.5 IFF Data: %ld\n", offset);
				return offset;
		    }
		}
	}
	Close(fh);	
    //Printf("Current Offset: %ld\n", offset);

	return offset;
}

char *Check_IFFType(char *srcename, LONG iffoffset)
{
    //Printf("\nCheck IFFType...\n");    
	
    //See: http://krashan.ppa.pl/articles/amigaicons/

	//78[diskobject]+(56)[drawerdata]=0?
    //20[imageheader]
    //2bpl 46~16=48x46=552[((w~16)*h)*bitplanes/8]
    //20[imageheader]
    //2bpl 46~16=48x46=552[((w~16)*h)*bitplanes/8]
	//[deftool]=0?
	//[tooltypes]=0?
    //FORM+4+ICONFACE

	//See: https://wiki.amigaos.net/wiki/IFFParse_Library

	//Remember to open the IFFParse.Library before using IFFParse code.
	//#define IFFParseBase	DemandOpenLibrary((struct Library **)&IconBase->ib_IFFParseBase, "iffparse.library", IFFParseBase_version)

	/* Code from AROS Icon Library */
	static const LONG stopchunks[] =
    {
        ID_ICON, ID_FACE,
        ID_ICON, ID_IMAG,        
        ID_ICON, ID_ARGB,
    };

	struct IFFHandle 	    *iff;
    struct Hook     	     iffhook;
    struct FileFaceChunk     fc;
    struct FileImageChunk    ic1, ic2;
    //UBYTE                   *argb1data = NULL, *argb2data = NULL;
    //UBYTE   	    	    *img1data = NULL, *img2data = NULL;
    //struct ColorRegister    *img1pal = NULL, *img2pal = NULL;
    BOOL  have_face = FALSE, have_imag1 = FALSE, have_imag2 = FALSE;
    BOOL                     have_argb1 = FALSE, have_argb2 = FALSE;
    //int have_png = 0;
    //LONG here, extrasize;
    APTR data;
	//UBYTE *dataBuffer;

	iffhook.h_Entry    = (HOOKFUNC)HookEntry;
    iffhook.h_SubEntry = (HOOKFUNC)MyMemStreamHandler;
    iffhook.h_Data     = (APTR)IconBase;

    
    //Printf("ReadIcon35 - Filename: %s\n", srcename);

	//if (IFFParseBase == NULL)
    	//return FALSE;

	//Printf ("ReadIcon35 - ReadIFF Working!\n");

	/* Get total icon file size */
	LONG imSize = GetFileSize(srcename);
	//Printf("Temp Icon FileSize: %ld\n", imSize);
	//unsigned char buffer[imSize];
	data = (UBYTE *)AllocVec(imSize-iffoffset, MEMF_ANY);    


    /* Open icon file to Read */
    BPTR fh = Open(srcename, MODE_OLDFILE);
    if (fh)
	{		
		//iffoffset = 1222;
		//**Seek Offset ??
		Seek(fh,iffoffset,OFFSET_BEGINNING); 
		Read(fh,data,imSize-iffoffset);

        //Using IFFParse_Library		
		if (iff = AllocIFF())
		{
			LONG error;
			//Printf("ReadIcon35. AllocIFF okay\n");			
    	    //iff->iff_Stream = (IPTR)fh; //Can be BPTR or pointer to data buffer
			iff->iff_Stream = (IPTR)&data;
			//iff->iff_Stream = Open(srcename, MODE_OLDFILE);

			//iff->iff_Stream = (ULONG) IDOS->Open ("filename", MODE_OLDFILE);
            //if(iff->iff_Stream) IIFFParse->InitIFFasDOS (iff);  /* use internal DOS stream manager */

			//Note: When using a custom handle, or a device other than the clipboard, you must provide your own flags and stream handler:
			//iff->iff_Stream = (ULONG) OpenMyCustomStream("foo");
            //IIFFParse->InitIFF (iff, IFFF_FSEEK | IFFF_RSEEK, &mystreamhook);
	
	        //Printf("ReadIcon35. InitIFF as iffHook\n");
	        InitIFF(iff, IFFF_RSEEK, &iffhook);
			//InitIFFasDOS(iff);			

            //After your stream is initialized, call OpenIFF():
            //error = IIFFParse->OpenIFF (iff, IFFF_READ);
			//Once you establish a read/write mode (by passing IFFF_READ or IFFF_WRITE), you remain in that mode until you call CloseIFF().

			//See: https://wiki.amigaos.net/wiki/Parsing_IFF

			//You can instruct the parser to stop when it encounters a specific IFF chunk by using the function StopChunk():
			//error = IIFFParse->StopChunk(iff, ID_ILBM, ID_BODY);

            //Printf("ReadIcon35. Try to OpenIFF\n");
			//if ((error = OpenIFF(handle, IFFF_READ)) == 0)			
			if (!OpenIFF(iff, IFFF_READ))			
			{				
                //Printf("ReadIcon35. OpenIFF okay\n"); 
				//if (!StopChunks(iff, stopchunks, 4))
				if (!StopChunks(iff, stopchunks, 3))
				//if (!StopChunk(iff, ID_ICON, ID_FACE))
				//if ((error = StopChunk(handle, ID_ICON, ID_FACE)) == 0)
				//if (!StopChunks(iff, stopchunks, 3))
				{							
	    	        //Printf("ReadIcon35. StopChunks okay\n");

                    //if ((error = OpenIFF(handle, IFFF_READ)) == 0)
					while(!(error = ParseIFF(iff, IFFPARSE_SCAN)))
					{
						struct ContextNode *cn;		    
		                cn = CurrentChunk(iff);

						//You can obtain a pointer to the current ContextNode through the function CurrentChunk():

						//struct ContextNode {
                        //struct MinNode  cn_Node;
                        //LONG            cn_ID;
                        //LONG            cn_Type;
                        //LONG            cn_Size;        /*  Size of this chunk             */
                        //LONG            cn_Scan;        /*  # of bytes read/written so far */
                        /*  There are private fields hiding here.  */
                        //};

    	    	        //Printf("inside ParseIFF loop\n"); //IFFParser stops at first Chunk: 'ID_FACE' 
						if ((cn->cn_ID == ID_FACE) && (cn->cn_Size >= sizeof(struct FileFaceChunk)))
		                {
							//When the parser encounters the requested chunk (stopchunk), parsing will stop, and ParseIFF() will return the value zero. 
							//The stream will be positioned ready to read the first data byte in the chunk. You may then call ReadChunkBytes() or 
							//ReadChunkRecords() to pull the data out of the current chunk.

							//You may call StopChunk() any number of times for any number of different chunk types. If you wish to identify the chunk on which you’ve stopped, 
							//you may call CurrentChunk() to get a pointer to the current ContextNode, and examine the cn_Type and cn_ID fields.
                            //Using StopChunk() for every chunk, you can parse IFF files in a manner very similar to the way you’re probably doing it now with manual methods.

		    	            //Printf("ReadIcon35. Found FACE chunk\n");
		    	            if (ReadChunkBytes(iff, &fc, sizeof(fc)) == sizeof(fc))
			                {
			                    have_face = TRUE;
			                }
						}
						else if (have_face && (cn->cn_ID == ID_IMAG) && (cn->cn_Size >= sizeof(struct FileImageChunk)))
						{
							//Printf("ReadIcon35. Found IMAG chunk\n");
							if (ReadChunkBytes(iff, (have_imag1 ? &ic2 : &ic1), sizeof(ic1)) == sizeof(ic1))
			                {
			                    if (have_imag1)
			                    {
			    	                //if (ReadImage35(&icon->ni_DiskObject, iff, &fc, &ic2, &img2data, &img2pal, IconBase))
				                    //{
			    	                    have_imag2 = TRUE;
				                    //}
			                    }
			                    else
			                    {
			    	                //if (ReadImage35(&icon->ni_DiskObject, iff, &fc, &ic1, &img1data, &img1pal, IconBase))
				                    //{
			    	                    have_imag1 = TRUE;
				                    //}
			                    }			    
			                }
						}
						else if (cn->cn_ID == ID_ARGB)
						{
							//Printf("ReadIcon35. Found ARGB chunk\n");
							if (have_argb1)
                            {
                                //if (ReadARGB35(&icon->ni_DiskObject, iff, &fc, cn->cn_Size, &argb2data, IconBase))
                                //{
                                    have_argb2 = TRUE;
                                //}
                            }
                            else
                            {
                                //if (ReadARGB35(&icon->ni_DiskObject, iff, &fc, cn->cn_Size, &argb1data, IconBase))
                                //{
                                have_argb1 = TRUE;
                                //}
			                }
						}
						//else if (cn->cn_ID == ID_png)
						//{
							//if (have_png < 2) 
							//{
                                    //LONG here = data - icon->ni_Extra.Data;
                                    //Printf("[%s] Found PNG image %d\n", __func__, have_png);
                                    //icon->ni_Extra.PNG[have_png].Offset = here;
                                    //icon->ni_Extra.PNG[have_png].Size = cn->cn_Size;
                            //} 
							//else 
							//{
                                //Printf("[%s] Ignoring PNG image %d\n", __func__, have_png);
                            //}
                            //have_png++;
						//}
						else 
						{
		                    //Printf("[%s] Unknown chunk ID 0x%08x\n", __func__, cn->cn_ID);
                        }			
		                //} // else if ((cn->cn_ID == ID_FACE)

					} /* while(!ParseIFF(iff, IFFPARSE_SCAN)) */

				} /* if (!StopChunks(iff, stopchunks, 4)) */				
				CloseIFF(iff);
			} /* if (!OpenIFF(iff, IFFF_READ)) */
			//if (iff->iff_Stream) Close (iff->iff_Stream);			
			FreeIFF(iff);			

		} /* if ((iff = AllocIFF())) */

		/* Only use the Width and Height fields of a FACE section
        * if it also contains image data. */
	}
	Close(fh);
	//FreeVec(data);

	/* Chech return types for icon images */
	//have_imag1 = TRUE; && have_imag2 = TRUE;
	//have_argb1 = TRUE; && have_argb2 = TRUE;
	//if (have_argb1 == TRUE && have_argb2 == TRUE)
	//{
		//if (have_imag1 == TRUE && have_imag2 == TRUE)
		    //return "argb";
		//else if (have_imag1 == FALSE && have_imag2 == FALSE)
		    //return "os41";
	//}

	/* Only check for the first image */
	if (have_argb1 == TRUE)
	{
		if (have_imag1 == TRUE)
		    return "argb";
		else if (have_imag1 == FALSE)
		    return "os41";
	}	
	else if (have_imag1 == TRUE && have_imag2 == TRUE)
	{
        return "os35";
	}
	        

	return "none";
}

/*********************************************************************************************/

void Display_Rectangle(char *color, char *position, WORD deltasize, struct Window *win)
{			
	LONG newpen;
	UWORD srce_x, srce_y;
	UWORD center_x, center_y;
    int width, height, depth;
	LONG x1, y1, x2, y2;
    struct RastPort *rastPort = win->RPort;	


	Printf ("Display Rectangle: %s\n\n", position);

	/* GET COLOR */

	if(stricmp(color,"red") == 0)
	{
        //newpen = ObtainBestPenA(win->WScreen->ViewPort.ColorMap, 0, 0, 0, 0); //black
		newpen = ObtainBestPenA(win->WScreen->ViewPort.ColorMap, 0xffffffff, 0x00000000, 0x00000000, 0); //red
		SetAPen(rastPort, newpen);
	}

    /* DRAW COLORED RECTANGLE */

	/* Centered ? */
	center_x = ((80-width)/2);
	center_y = ((80-height)/2);

    /* Get coordinates */		
	if(stricmp(position,"left") == 0)
	{
		/* Top left of First Raised Panel */	    
		srce_x = 22 + center_x + deltasize;
	    srce_y = 25 + topoffset + center_y + deltasize;

		/* Erase Display Area Left */ //EraseRect( rp, xmin, ymin, xmax, ymax) a1  d0:16 d1:16 d2:16 d3:16
		//EraseRect(win->RPort, leftoffset + 13 + 1 + deltasize, win->BorderTop + topoffset + 31 + deltasize,
        //win->Width - 109 - win->BorderRight - deltasize,
        //win->Height - 50 - win->BorderBottom - deltasize);

		x1 = leftoffset + 13 + 1 + deltasize; //xmin
		//y1 = win->BorderTop + topoffset - 3 + deltasize; //ymin
		y1 = win->BorderTop + topoffset +31 + deltasize; //ymin
		x2 = win->Width - 109 - win->BorderRight - deltasize; //xmax
		y2 = win->Height - 50 - win->BorderBottom - deltasize; //ymax
		Printf ("Display Image %s x1= %d\n", position, x1);
		Printf ("Display Image %s y1= %d\n", position, y1);				
		Printf ("Display Image %s x2= %d\n", position, x2); //x1+77 //100-34=+66
		Printf ("Display Image %s y2= %d\n", position, y2); //y1+77 //156-126=-30				

		/* Draw Left Rectangle */
		Move(rastPort, x1, y1); Draw(rastPort, x1+77, y1);
		Move(rastPort, x1+77, y1); Draw(rastPort, x1+77, y1+77);
		Move(rastPort, x1, y1); Draw(rastPort, x1, y1+77);
		Move(rastPort, x1, y1+77); Draw(rastPort, x1+77, y1+77);      
       
	}
	else if(stricmp(position,"right") == 0)
	{
		/* Top left of Second Raised Panel */
		srce_x = 111 + center_x + deltasize;
	    srce_y = 25 + topoffset + center_y + deltasize;

		/* Erase Display Area Right */
		//EraseRect(win->RPort, leftoffset + 102 + 1 + deltasize, win->BorderTop + topoffset + 31 + deltasize,
        //win->Width - 20 - win->BorderRight - deltasize,
        //win->Height - 50 - win->BorderBottom - deltasize);
        
        x1 = leftoffset + 102 + 1 + deltasize; //xmin
		//y1 = win->BorderTop + topoffset - 3 + deltasize; //ymin
		y1 = win->BorderTop + topoffset +31 + deltasize; //ymin
		x2 = win->Width - 20 - win->BorderRight - deltasize;
		y2 = win->Height - 50 - win->BorderBottom - deltasize;
		Printf ("Display Image %s x1= %d\n", position, x1);
		Printf ("Display Image %s y1= %d\n", position, y1);				
		Printf ("Display Image %s x2= %d\n", position, x2);
		Printf ("Display Image %s y2= %d\n", position, y2);				 

		/* Draw Right Rectangle */
		Move(rastPort, x1, y1); Draw(rastPort, x1+77, y1);
		Move(rastPort, x1+77, y1); Draw(rastPort, x1+77, y1+77);
		Move(rastPort, x1, y1); Draw(rastPort, x1, y1+77);
		Move(rastPort, x1, y1+77); Draw(rastPort, x1+77, y1+77);
	}
	ReleasePen(win->WScreen->ViewPort.ColorMap, newpen);	
    
	//return 0;
}

/*********************************************************************************************/

//BOOL Display_Icon(char *srcename, char *position, WORD deltasize, struct Window *win)
BOOL Display_Icon(char *srcename, struct DiskObject *icon, char *position, WORD deltasize, struct Window *win)
{
	//Printf ("Display Icon: %s\n", position); 

	int bpp;
	//int index;	
	char *type = NULL;		
	UBYTE *pixelBuffer;
	BOOL retval = TRUE;
	//BOOL isOS35 = FALSE;
	//BOOL isARGB = FALSE;
    LONG x1, y1, x2, y2;
	UWORD srce_x, srce_y;
	UWORD center_x, center_y;
    int width, height, depth;	
	struct IconInfo info = {0};		
    //struct BitMapHeader *bmhd;	
	//struct DiskObject *icon = NULL;

    //Printf ("ReadBmhd_by_datatype");
	//bmhd = readbmhd(srcename);
	//width = bmhd->bmh_Width; 
	//height = bmhd->bmh_Height; 
	//depth = bmhd->bmh_Depth;
	//depth = 32;
	//bpp = (depth/8);

	//Printf ("Display Icon Filename: %s\n", srcename);	


	//**Only concerned with checking & displaying OS3.5 or PNG**//
	//By now ARGB will have been converted to PNG.
	//But we will check everything again to be sure.
	//type = Check_Icon_Signature(srcename);
	//if PNG or IFF use IconControl to get images.

	//if(type == IFF && ifftype != none) || type == PNG)
	//use IconControl to get images.
	//if(type == IFF) get ARGB data.



    /* Get Disk Object for Display Purposes */
	//srcefile = RemoveFileExtension(srcename);
	//icon = GetDiskObject(srcefile);
	//icon = GetDiskObjectNew(srcefile);

	if (!icon) 
	{
		/* Show Output Message for Icon Load Error */
		STRPTR outputmsg = "Loading Icon Failed. Bad Filename.";
		OutputMessage(outputmsg, win);
		//Cleanup(); //EraseRectangle
		return FALSE;
		//Printf ("Cannot GetDiskObject: %s\n", srcename);	    
	}

    //LONG nic = 0, pal = 0, new = 0;
	LONG pal = 0, new = 0;
	IconControl(icon,
	    //ICONCTRLA_GetImageDataFormat,	&info.Format,
		ICONCTRLA_IsPaletteMapped,      &pal,		
		ICONCTRLA_IsNewIcon,            &new,
		ICONCTRLA_GetWidth,			    &info.Width,
		ICONCTRLA_GetHeight,			&info.Height,
		TAG_END);

	//index = 0;	
	depth = 32;
	bpp = (depth/8);
	width = info.Width;
	height = info.Height;

	//Printf ("Dimensions: %d, %d\n", info.Width, info.Height);
    
	//See: ProcessIcon.c - IconControl(dobj, ICONCTRLA_IsPaletteMapped, &p, ICONCTRLA_IsNewIcon, &n, TAG_DONE);
    //if(p) type = TYPE_OS35;
	//if(pal) Printf ("IFFType: OS35\n");
    //if(n) type = TYPE_NEWICON35;
	//if(new) Printf ("IFFType: NEWICON35\n");
    //if(p && n) type = TYPE_BOTHCOLOR;
	//if(pal && new) Printf ("IFFType: BOTH COLOR\n");
	//if(!pal && nic) Printf ("IFFType: OS35 ARGB\n");

	//See: Icon Library - IconControl.c
	//BOOL ICONCTRLA_IsPaletteMapped, _IsNativeIcon
	//ARGB32 format (ULONG **)ICONCTRLA_GetARGBImageData1
	//ARGB32 format (ULONG **)ICONCTRLA_GetARGBImageData2
    
	//case ICONCTRLA_GetWidth:
	    //if (nativeicon && nativeicon->ni_Face.Width > 0)
		//{
		    //STORE((ULONG *)tag->ti_Data, nativeicon->ni_Face.Width);
		    //processed++;
		//}

	//case ICONCTRLA_GetHeight:
	    //if (nativeicon)
		//{
		    //STORE((ULONG *)tag->ti_Data, nativeicon->ni_Face.Height);
		    //processed++;
		//}

    //case ICONCTRLA_GetARGBImageData1:
	    //if (image1)
		//{
		    //FetchIconARGB(icon, 0);
		    //STORE((CONST ULONG **)tag->ti_Data, image1->ARGB);
		//}

	/* Check Icon Image Depth */
	//if(depth != 32) return FALSE;

    /* Check Icon Image Size */
	//if(width > 80 || height > 80) return FALSE;

    /* Check Icon Image Type */
	//const char *dtypename = DetectBaseName_by_datatype(srcename);
	//if( stricmp(dtypename, "png") !=0 ) return FALSE;

    //pixelBuffer = (UBYTE *)AllocVec(width * height * 4, MEMF_ANY);
    pixelBuffer = (UBYTE *)AllocVec(width * height * bpp, MEMF_ANY);

    
	/* GET ICON TYPE */

	//PNG or IFF - Read File Signature 'png' or 'e310'
	//type = Check_Icon(srcename);
	type = Check_Icon_Signature(srcename);
	//png = DualPNG icon file
    //iff = Classic icon with or without ARGB. If hybrid return ARGB pixel data.
	//Printf ("IconClass: %s\n", type);


    /* GET ICON IMAGE */   
	
	if(stricmp(type, "png")==0)
	{
		/* Load First Icon Image by PNG Datatype */
        //Printf ("ReadPixels_by_datatype\n");		
	    //retval = readpixels_by_datatype(pixelBuffer, srcename, 4);
		retval = ReadPixels_by_datatype(pixelBuffer, srcename, bpp);

		//Draw_Rectangle will draw colored rectangles to provide visual clues to what icon style is loaded.
        //Graphics Libary drawing routine is used after SetAPen to set the color. Color = Red or White.
		if(stricmp(position, "left")==0)
		{
			srceicontype = type;
		}
		else if(stricmp(position, "right")==0)
		{
			desticontype = type;
		}	
	}
	else if(stricmp(type, "iff")==0)
	{
		/* Return ARGB pixel data from OS35 Hybrid or OS41 Icon */

		ULONG *data1;
	    ULONG *data2;
		//BPTR file = NULL;	
		//Test Writing Hybrid Disk Object		
		//PutDiskObject("Ram Disk:TestIcon", icon);

		/* Find Beginning of OS3.5 IFF Data */
		LONG iffoffset = Get_IFFOffset(srcename); //Find offset for OS3.5 IFF data		
		//Printf ("\nReturned IFF Offset: %ld\n", iffoffset);
		/* Check for OS3.5 & ARGB Data Chunks */
	    char *ifftype = Check_IFFType(srcename, iffoffset); //OS35/OS41 or ARGB or NONE
		//Printf ("IFF Icon Type: %s\n", ifftype);

		//Draw_Rectangle will draw colored rectangles to provide visual clues to what icon style is loaded.
        //Graphics Libary drawing routine is used after SetAPen to set the color. Color = Red or White.
		if(stricmp(position, "left")==0)
		{
			srceicontype = ifftype;
		}
		else if(stricmp(position, "right")==0)
		{
			desticontype = ifftype;
		}


        /* Get Image Data from ARGB Data Chunks */		
        if(stricmp(ifftype, "argb")==0)
		{		    
	        IconControl(icon,			    	    
		        ICONCTRLA_GetARGBImageData1,      &data1,
		        ICONCTRLA_GetARGBImageData2,      &data2,
		        TAG_END);
			
			pixelBuffer = (UBYTE *)data1; /* 1st Image For Icon Display */


			/* See: ReadMemPNG for method to convert ULONG array to UBYTE array */
			/* ReadMemPNG can be found in "diskobjPNGio.c" inside Icon Library */


			/* If position = "left" check icon type. 
	        If type = "argb" convert to png with messagebox.
            Save png icon as "Ram Disk:SourceIcon.info" */

			if(stricmp(position, "left")==0)
			{
				/* Write datatype object to file */
		        //if (file = Open("Ram Disk:Im1.png", MODE_NEWFILE))
		        //{
                    //Seek(file,0,OFFSET_BEGINNING);
                    //Write(file,pixelBuffer,(width * height * bpp));
				//}
                //if (file) Close (file);

				//Printf ("Save Both ARGB Images as PNG\n");

			    /* Save Both Images as PNG */
			    char *image1 = "Ram Disk:Im1.png";
			    //SavePixels_by_datatype((UBYTE *)data1, image1, "png", 4);
			    retval = SavePixels_by_datatype(data1, image1, width, height);				
			    char *image2 = "Ram Disk:Im2.png";
			    //SavePixels_by_datatype((UBYTE *)data2, image2, "png", 4);
			    //SavePixels_by_datatype((UBYTE *)data2, image2, info, bpp);
				retval = SavePixels_by_datatype(data2, image2, width, height);

                /* Make PNG Icon from Image Data for Future Use */
			    //void Make_Icon(char *image1, char *image2, char *outfile);
			    //Make_Icon(image1, image2, "Ram Disk:SourceIcon.info");
			
			    /* Delete Images */
			    //DeleteFile(image1);
			    //DeleteFile(image2);
			}
			//Save_Icon(srcedobj, outputname); //Save_Icon(destdobj, outputname);						
		}
		else if(stricmp(ifftype, "os41")==0)
		{
            IconControl(icon,			    	    
		        ICONCTRLA_GetARGBImageData1,      &data1,
		        ICONCTRLA_GetARGBImageData2,      &data2,
		        TAG_END);
			
			pixelBuffer = (UBYTE *)data1; /* 1st Image For Icon Display */

			if(stricmp(position, "left")==0)
			{
				//Printf ("Save Both ARGB Images as PNG\n");
								
			    /* Save Both Images as PNG */
			    char *image1 = "Ram Disk:Im1.png";
			    //SavePixels_by_datatype((UBYTE *)data1, image1, "png", 4);
			    retval = SavePixels_by_datatype(data1, image1, width, height);				
			    char *image2 = "Ram Disk:Im2.png";
			    //SavePixels_by_datatype((UBYTE *)data2, image2, "png", 4);
			    //SavePixels_by_datatype((UBYTE *)data2, image2, info, bpp);
				retval = SavePixels_by_datatype(data2, image2, width, height);

                /* Make PNG Icon from Image Data for Future Use */
			    //void Make_Icon(char *image1, char *image2, char *outfile);
			    //Make_Icon(image1, image2, "Ram Disk:SourceIcon.info");
			
			    /* Delete Images */
			    //DeleteFile(image1);
			    //DeleteFile(image2);
			}
		}
		else if(stricmp(ifftype, "os35")==0)
		{
			/* Get Image Data from OS3.5 IFF Data */
			//See INFO Datatype for Method for getting Image Data.
			/* Use switch (index) instead of assuming first image index */

			IconControl(icon,			    
				ICONCTRLA_GetWidth,			    &info.Width,
				ICONCTRLA_GetHeight,			&info.Height,
				ICONCTRLA_GetPaletteSize1,		&info.NumColors,
				ICONCTRLA_GetPalette1,			&info.CMap,
				ICONCTRLA_GetImageData1,		&info.Image,
				TAG_END);

            //Printf ("ReadOS35_by_datatype\n");		
		    /* Return 32bit Pixel Data from OS35 Image Data */			
			//retval = ReadOS35_by_datatype(pixelBuffer, info, bpp);		
		    //retval = ReadOS35_by_datatype(pixelBuffer, info.Image, info.CMap, 4);
			//BOOL SaveAsDatatype_lut8(UBYTE *pixels, UBYTE *colorBytes, char *file_name, char *type);			

			/* Write image data to file */
		    //if (file = Open("Ram Disk:Im1.bin", MODE_NEWFILE))
		    //{
                //Seek(file,0,OFFSET_BEGINNING);
                //Write(file,info.Image,(info.Width * info.Height));
			//}
            //if (file) Close (file);

			/* New function for glow icon images */
			//Printf ("ReadOS35_by_datatype - ReadImage.\n");	        
			retval = ReadImage (pixelBuffer, info, bpp);
		}				
	}
	

    /* SHOW ICON IMAGE */

	/* Centered ? */
	center_x = ((80-width)/2);
	center_y = ((80-height)/2);

    /* Get coordinates */		
	if(stricmp(position,"left") == 0)
	{
		/* Top left of First Raised Panel */
	    //srce_x = 26 + center_x + deltasize;
		srce_x = 22 + center_x + deltasize;
	    srce_y = 25 + topoffset + center_y + deltasize;

        x1 = leftoffset + 13 + 1 + deltasize; //xmin
		y1 = win->BorderTop + topoffset + 1 + deltasize; //ymin
		x2 = win->Width - 145 - win->BorderRight - deltasize; //xmax
		y2 = win->Height - 20 - win->BorderBottom - deltasize; //ymax
		//Printf ("Display Image %s x1= %d\n", position, x1);
		//Printf ("Display Image %s y1= %d\n", position, y1);				
		//Printf ("Display Image %s x2= %d\n", position, x2); 
		//Printf ("Display Image %s y2= %d\n", position, y2);

		/* Erase Icon Image Left */ 
		EraseRect(win->RPort, leftoffset + 13 + 1 + deltasize, win->BorderTop + topoffset + 1 + deltasize,
        win->Width - 145 - win->BorderRight - deltasize,
        win->Height - 20 - win->BorderBottom - deltasize);
	}
	else if(stricmp(position,"right") == 0)
	{
		/* Top left of Second Raised Panel */
	    //srce_x = 151 + center_x + deltasize;
		srce_x = 147 + center_x + deltasize;
	    srce_y = 25 + topoffset + center_y + deltasize;

        x1 = leftoffset + 138 + 1 + deltasize; //xmin
		y1 = win->BorderTop + topoffset + 1 + deltasize; //ymin
		x2 = win->Width - 20 - win->BorderRight - deltasize; //xmax
		y2 = win->Height - 20 - win->BorderBottom - deltasize; //ymax
		//Printf ("Display Image %s x1= %d\n", position, x1);
		//Printf ("Display Image %s y1= %d\n", position, y1);				
		//Printf ("Display Image %s x2= %d\n", position, x2); 
		//Printf ("Display Image %s y2= %d\n", position, y2);

		/* Erase Icon Image Right */
		EraseRect(win->RPort, leftoffset + 138 + 1 + deltasize, win->BorderTop + topoffset + 1 + deltasize,
        win->Width - 20 - win->BorderRight - deltasize,
        win->Height - 20 - win->BorderBottom - deltasize);
	}	
	

	//if(retval == TRUE)    
	if(depth == 32)
	{
		/* Display 32bit icon image */
		//Printf ("WritePixelArrayAlpha\n");	        
		WritePixelArrayAlpha(pixelBuffer, 0, 0, width * depth / 8, win->RPort, srce_x, srce_y, width, height, RECTFMT_ARGB);
	}
	else if(depth == 24)
	{
		/* Display 24bit icon image */
        //Printf ("WritePixelArray\n");	        
		WritePixelArray(pixelBuffer, 0, 0, width * depth / 8, win->RPort, srce_x, srce_y, width, height, RECTFMT_RGB);
	}

	/* Check for icon style mismatch */
	if(srceicontype != NULL && desticontype != NULL)
	{
	    if(stricmp(srceicontype, desticontype)!=0)
	    {
		    //Printf ("**Warning! Icon Style Mismatch.**\n");
            /* Show MessageBox ?? */  //PNG, OS41, OS35, ARGB
			/* "Source icon style is ----. Destination icon style is ----. 
			    If you continue the Destination icon style will be changed." */

			/* Show Output Message for Icon Style Mismatch */
			STRPTR outputmsg = "If you continue the Destination icon style will be changed.";
			//STRPTR outputmsg = "Icon style mismatch! If you exchange\n Destination icon style will be changed."
			OutputMessage(outputmsg, win);			

		    //Draw_Rectangle(win, "White", "left"); //srceicontype //White or Red
		    //Draw_Rectangle(win, "red", "right"); //desticontype //White or Red
	    }
	}	
		
	//if (pixelBuffer) FreeVec(pixelBuffer);
	//FreeDiskObject(icon);

    return retval;
	//return TRUE;
}

/*********************************************************************************************/

const char  *DetectBaseName_by_datatype(char *file_name)
{
    struct DataType *dt;
    const char *basename;    
	Object *DTImage = NULL;		

	DTImage = NewDTObject(	file_name, (DTA_SourceType), DTST_FILE, TAG_DONE);

	if (DTImage)
	{
        /* Get DataType BaseName */        
        GetDTAttrs(DTImage, DTA_DataType, (IPTR)&dt, TAG_DONE );        
        basename = dt->dtn_Header->dth_BaseName;
    }
    ReleaseDataType(dt);
    DisposeDTObject(DTImage);

    return basename;
}

/**************************************************************************************************/

struct BitMapHeader *readbmhd(char *file_name)
{
	Object *DTImage = NULL;
	struct BitMapHeader *bmh;

	DTImage = NewDTObject(	file_name,
							(DTA_SourceType),	DTST_FILE,
							(DTA_GroupID),		GID_PICTURE,
							(PDTA_Remap),		FALSE,
							(OBP_Precision),	PRECISION_EXACT,
							TAG_DONE);

	if (DTImage)
	{

		if (GetDTAttrs( DTImage,
					PDTA_BitMapHeader, &bmh,
					TAG_END ) == 1)

		{            
            return bmh;
        }
    }
    DisposeDTObject( DTImage );
    return bmh;
}

/**************************************************************************************************/

//BOOL readpixels_by_datatype(UBYTE *pixels, char *file_name, int bpp)
BOOL ReadPixels_by_datatype(UBYTE *pixels, char *file_name, int bpp)
{	
    Object *DTImage = NULL;
	struct BitMapHeader *bmh;
	struct pdtBlitPixelArray bpa;

	DTImage = NewDTObject(	file_name,
							(DTA_SourceType),	DTST_FILE,
							(DTA_GroupID),		GID_PICTURE,
							(PDTA_Remap),		FALSE,
							(OBP_Precision),	PRECISION_EXACT,
							TAG_DONE);

	if (DTImage)
	{

		if (GetDTAttrs( DTImage,
					PDTA_BitMapHeader, (IPTR)&bmh,
					TAG_END ) == 1)

		{					
			if (pixels)
			{
				bpa.MethodID = PDTM_READPIXELARRAY;
				bpa.pbpa_PixelData = (APTR)pixels;
				bpa.pbpa_PixelFormat = (bpp = 4) ? PBPAFMT_ARGB: PBPAFMT_RGB;
				bpa.pbpa_PixelArrayMod = bmh->bmh_Width * bpp;
				bpa.pbpa_Left = 0;
				bpa.pbpa_Top = 0;
				bpa.pbpa_Width = bmh->bmh_Width;
				bpa.pbpa_Height = bmh->bmh_Height;

				DoMethodA( DTImage, (Msg)&bpa );
				DisposeDTObject( DTImage );
				return TRUE;
			}
		}
		DisposeDTObject( DTImage );
	}
	return FALSE;
}

/**************************************************************************************************/

//BOOL ReadOS35_by_datatype(UBYTE *pixels, struct IconInfo info, int bpp)
BOOL ReadImage (UBYTE *pixels, struct IconInfo info, int bpp)
{
	int i;	
	int numcolors;	
	Object *DTImage = NULL;	
	struct BitMapHeader *bmh = NULL;
	struct pdtBlitPixelArray bpa; 
    struct pdtBlitPixelArray dtb;
    struct ColorRegister *cmap;
    ULONG *cregs;	
    
    //Printf ("ReadOS35_by_datatype: ReadImage.\n");
	//Printf ("Reading OS35 Image...\n");

	/* Set type for PNG */
    char *type = "png";

    DTImage = NewDTObject(	(APTR)NULL,
            DTA_SourceType, DTST_RAM,
            DTA_BaseName, (IPTR)type,
            PDTA_DestMode, PMODE_V43,
            TAG_DONE);
	
    GetDTAttrs(DTImage,
		PDTA_BitMapHeader,	&bmh,		
		TAG_END);
	
    
	bmh->bmh_Depth = 8;
	bmh->bmh_Width = info.Width;
	bmh->bmh_Height = info.Height;
	numcolors = info.NumColors;	
    

	//Printf("Picture Data: Width %d x Height %d\n", bmh->bmh_Width, bmh->bmh_Height));		
	//Printf("Picture Data: BitDepth: %d\n", bmh->bmh_Depth));	
	//Printf("Picture Data: NumColors: %d\n", info.NumColors));

	//Printf ("Dimensions: (w,h,d) %d, %d, %d\n", bmh->bmh_Width, bmh->bmh_Height, bmh->bmh_Depth);

    	
	if (!info.CMap) 
    {        
		return FALSE;				
    }

	if(bmh->bmh_Depth == 8)
    {
        /* Get cregs and color palette */
        GetDTAttrs( DTImage,
            PDTA_CRegs,        (IPTR)&cregs,			
			PDTA_ColorRegisters,     (IPTR)&cmap,                                       
            TAG_END );        
    }

	/* Set colorregs & ColorRegister */

	for (i = 0; i < info.NumColors; i++) 
	{
		cmap->red = info.CMap->red;
		cmap->green = info.CMap->green;
		cmap->blue = info.CMap->blue;
		*cregs++ = RGB8to32(info.CMap->red);
		*cregs++ = RGB8to32(info.CMap->green);
		*cregs++ = RGB8to32(info.CMap->blue);
		cmap++; info.CMap++;
	}   
	
    
    if (!info.Image) 
    {        
		return FALSE;				
    }    

	/* Copy LUT8 Pixel Data to New DataType Object */
    if ( DTImage )
	{        
		dtb.MethodID = PDTM_WRITEPIXELARRAY;
		dtb.pbpa_PixelData = (APTR)info.Image;        
		dtb.pbpa_PixelFormat = PBPAFMT_LUT8;
		dtb.pbpa_PixelArrayMod = bmh->bmh_Width;
		dtb.pbpa_Left = 0;
		dtb.pbpa_Top = 0;
		dtb.pbpa_Width = bmh->bmh_Width;
		dtb.pbpa_Height = bmh->bmh_Height;
		
		DoMethodA(DTImage, (Msg) &dtb);
    }

	/* Set datatype attributes */	

	SetDTAttrs(DTImage, NULL, NULL,	    
		DTA_NominalHoriz,	bmh->bmh_Width,
		DTA_NominalVert,	bmh->bmh_Height,
		PDTA_NumColors,     numcolors,
		TAG_END);


	/* Save Image as PNG */
	//char *image1 = "Ram Disk:Im1.png";

	/* Write datatype object to file */
	//if (file = Open(image1, MODE_NEWFILE))
	//{
		//dtw.MethodID = DTM_WRITE;
		//dtw.dtw_GInfo = NULL;
		//dtw.dtw_FileHandle = file;
		//dtw.dtw_Mode = DTWM_RAW;
		//dtw.dtw_AttrList = NULL;

		//if (!DoMethodA(DTImage, (Msg) &dtw)) return FALSE;
	//}
	//if (file) Close (file);   
    
    
	/* Read ARGB Pixel Data from New DataType Object */
	if (pixels)
	{
		bpa.MethodID = PDTM_READPIXELARRAY;
		bpa.pbpa_PixelData = (APTR)pixels;
		bpa.pbpa_PixelFormat = (bpp = 4) ? PBPAFMT_ARGB: PBPAFMT_RGB;
		bpa.pbpa_PixelArrayMod = bmh->bmh_Width * bpp;
		bpa.pbpa_Left = 0;
		bpa.pbpa_Top = 0;
		bpa.pbpa_Width = bmh->bmh_Width;
		bpa.pbpa_Height = bmh->bmh_Height;

		DoMethodA( DTImage, (Msg)&bpa );		
	}

	DisposeDTObject( DTImage );

	
	return TRUE;
}

/**************************************************************************************************/

//BOOL SavePixels_by_datatype(UBYTE *pixels, char *filename, char *type, int bpp)
BOOL SavePixels_by_datatype(ULONG *data, char *filename, int width, int height)
{	
	UBYTE *pixels;
	BPTR file = NULL;
	BOOL retval = FALSE;
	Object *DTImage = NULL;
	struct dtWrite dtw;
	struct BitMapHeader *bmhd;	
	struct pdtBlitPixelArray dtb;

	//Printf("SavePixels_by_datatype: ARGB to PNG\n");

	char *type = "png";	
		
	DTImage = NewDTObject(	(APTR)NULL,
							DTA_SourceType, DTST_RAM,
							DTA_BaseName, (IPTR)type,
							PDTA_DestMode, PMODE_V43,
							TAG_DONE);
	if (!DTImage) return(FALSE);

	
	if (GetDTAttrs( DTImage,
					PDTA_BitMapHeader, (IPTR)&bmhd,
					TAG_END ) == 1)
	{
		bmhd->bmh_Width = width;
		bmhd->bmh_Height = height;
		bmhd->bmh_Left = 0;
        bmhd->bmh_Top = 0;
        bmhd->bmh_Depth = 32;
        bmhd->bmh_Masking = 0;
        bmhd->bmh_Compression = 0;
        bmhd->bmh_Pad = 0;
        bmhd->bmh_Transparent = 0;
        bmhd->bmh_XAspect = 10;
        bmhd->bmh_YAspect = 11;
        bmhd->bmh_PageWidth = width;
        bmhd->bmh_PageWidth = height;

		//Printf ("Icon Dimensions: %d, %d\n", bmhd->bmh_Width, bmhd->bmh_Height);

		pixels = (UBYTE *)AllocVec(width * height * 4, MEMF_ANY);
		pixels = (UBYTE *)data; /* ARGB Image Data to Convert */

		/* See: ReadMemPNG for method to convert ULONG array to UBYTE array */
		
		dtb.MethodID = PDTM_WRITEPIXELARRAY;
		dtb.pbpa_PixelData = (APTR)pixels;
		dtb.pbpa_PixelFormat = PBPAFMT_ARGB;
		dtb.pbpa_PixelArrayMod = bmhd->bmh_Width * 4;
		dtb.pbpa_Left = 0;
		dtb.pbpa_Top = 0;
		dtb.pbpa_Width = bmhd->bmh_Width;
		dtb.pbpa_Height = bmhd->bmh_Height;
		
		DoMethodA(DTImage, (Msg) &dtb);

		/* Write datatype object to file */
		if (file = Open(filename, MODE_NEWFILE))
		{
			dtw.MethodID = DTM_WRITE;
			dtw.dtw_GInfo = NULL;
			dtw.dtw_FileHandle = file;
			dtw.dtw_Mode = DTWM_RAW;
			dtw.dtw_AttrList = NULL;

			if (DoMethodA(DTImage, (Msg) &dtw)) retval = TRUE;
		}
	}

	if (file) Close (file);
	//if (pixels) FreeVec(pixels);
	if (DTImage) DisposeDTObject(DTImage);
	return retval;
}

/**************************************************************************************************/

void Copy_File_Append(char *srcename, char *destname, LONG offset, LONG filebytes)
{
    BPTR infile, outfile;
	UBYTE buffer[filebytes];    
    
    infile = Open(srcename, MODE_OLDFILE);
    if(infile)    
    {                
        Seek(infile,0,OFFSET_BEGINNING);
	    Read(infile,buffer,filebytes);
    }
    outfile = Open(destname, MODE_OLDFILE);
    if(outfile)
    {        
        Seek(outfile,offset,OFFSET_BEGINNING);
        Write(outfile,buffer,filebytes);               
    }	    

    if(infile) Close(infile);
    if(outfile) Close(outfile);
}

/**************************************************************************************************/

void Copy_Data(char *srcename, char *destname, LONG filebytes)
{
    BPTR infile, outfile;
	UBYTE buffer[filebytes];    
    
    infile = Open(srcename, MODE_OLDFILE);
    if(infile)    
    {                
        Seek(infile,0,OFFSET_BEGINNING);
	    Read(infile,buffer,filebytes);
    }
    outfile = Open(destname, MODE_OLDFILE);
    if(outfile)
    {        
        Seek(outfile,0,OFFSET_BEGINNING);
        Write(outfile,buffer,filebytes);               
    }	    

    if(infile) Close(infile);
    if(outfile) Close(outfile);
}

/**************************************************************************************************/

void Copy_File(char *srcename, char *destname, LONG filebytes)
{
    BPTR infile, outfile;
	UBYTE buffer[filebytes];    
    
    infile = Open(srcename, MODE_OLDFILE);
    if(infile)    
    {                
        Seek(infile,0,OFFSET_BEGINNING);
	    Read(infile,buffer,filebytes);
    }	
    outfile = Open(destname, MODE_NEWFILE);
    if(outfile)
    {        
        Seek(outfile,0,OFFSET_BEGINNING);
        Write(outfile,buffer,filebytes);               
    }	    

    if(infile) Close(infile);
    if(outfile) Close(outfile);
}

/**************************************************************************************************/

void SetFilesize(char *name, LONG newsize)
{
	BPTR file = Open(name, MODE_OLDFILE);
	SetFileSize(file,newsize,OFFSET_BEGINNING);
	Close(file);
}

/**************************************************************************************************/

LONG GetFileSize(char *imName)
{
    BPTR file = Open(imName, MODE_OLDFILE);
    struct FileInfoBlock *fib = (struct FileInfoBlock*)AllocDosObject( DOS_FIB, NULL );
    LONG size = -1;
    
    if (fib)
    {
	    if (ExamineFH( file, fib ))
	    {
	        if ((fib->fib_DirEntryType > 0) && (fib->fib_DirEntryType != ST_SOFTLINK))
	        {
		        size = 0;
	        }
	        size = fib->fib_Size;
	    }
	    FreeDosObject( DOS_FIB, fib );
    }
    Close(file);
    
    return size;
}

/**************************************************************************************************/

char *Check_Icon_Signature(char *srcename)
{	
	char *icontype;
	UBYTE fileID[5];
    
	//Printf("Read file signature to get type\n");
	if(srcename) 
	{
        BPTR file = Open(srcename, MODE_OLDFILE);
        if(file)
		{
            Seek(file,0,OFFSET_BEGINNING);
	        Read(file,fileID, 4);
	        fileID[4] = ('\0');

			if( fileID[1] == 'P' && fileID[2] == 'N' && fileID[3] == 'G' )
            {
				//Printf("Found PNG Icon File!\n");
                icontype = "png";				           
            }
			else if( fileID[0] == 0xE3 && fileID[1] == 0x10 )
            {
                //Printf("Found IFF Icon File!\n");            
                icontype = "iff";                
            }
		}
		Close(file);
	}    

    return icontype;
}

/**************************************************************************************************/

char *RemoveFileExtension(char *path) 
{
    /* This works */
    UBYTE *buffer;
    UBYTE *filePart;
    char *baseName;
    ULONG pathSize;
    ULONG partSize;
    ULONG position = 0;
    
	/* Remove ONLY ONE '.' segment in filename */
 
    pathSize = strlen(path) * sizeof(UBYTE *) + 1;
 
    /* Get the file part which we need to ignore. */
    filePart = FilePart(path);	
    partSize = strlen(filePart) * sizeof(UBYTE *) + 1;
 
    /* Allocate memory for the base name. */
    if((baseName = 
        AllocMem(
            strlen(path) * sizeof(UBYTE *) - strlen(filePart) + 1, 
            MEMF_ANY|MEMF_CLEAR
        )
    ) == NULL) return NULL;
 
    /* Allocate memory for path part. */
    if((buffer = AllocMem(pathSize, MEMF_ANY|MEMF_CLEAR)) == NULL) 
        return NULL;
 
    /* Strip pieces of the path and compose them. */
    do 
	{
        position = SplitName(path, '.', buffer, position, pathSize);
 
        switch(position == -1 ||
            strlen(buffer) == 0 || 
            strncmp(buffer, filePart, partSize) == 0) 
            {
 
            case TRUE: break; /* We have found the file part. */
            default: /* Otherwise, copy the buffer into the base path. */
                sprintf(baseName + strlen(baseName), "%s/", buffer);
                break;
            }
 
    } while(position != -1);
 
    switch(strlen(baseName)) 
    {
        case 0: /* Attempt to build a drive path. */
            if(SplitName(path, ':', buffer, 0, pathSize) != -1)
                sprintf(baseName + strlen(baseName), "%s:", buffer);
            break;
        default: /* Otherwise, strip trailing char. */
            baseName[strlen(baseName)-1] = '\0';
            break;
    }

	/* Restore dots in filename */	
	int i = 0;
	int count = 0;
    for(i = strlen(filePart) - 1;  i >= 0; i--)
	{
		if (filePart[i] == '.') 
		{
			count++;
			if(count > 1) 
		    {				
			    baseName[strlen(baseName)+1-i] = '.';
		    }
		}		
	}
 
    return baseName;
}

/**************************************************************************************************/

char *get_ext(char *filename)
{
	static char extension[32];
	int position=strlen((char *)filename)-1;

	strcpy(extension,"");

	while(position > -1 && filename[position] != '.') position--;

	if (position > -1)
	{
		strncpy(extension,&filename[position+1],32);
	}

	return extension;
}

/**************************************************************************************************/

char* GetFileNameFromPath(char* path)
{
    int i = 0;
    for(i = strlen(path) - 1;  i >= 0; i--)
    {
        //Includes Full Path with File Name
        if (path[i] == '/' && path[i] != ':')
        {
            return &path[i+1];
        }

        //Includes Volume + File Name Only!
        if (path[i] == ':' && path[i] != '/')
        {
            return &path[i+1];
        }
    }

   return path;
}

/**************************************************************************************************/

char *GetPath(char *path) 
{
    /* This works */
    UBYTE *buffer;
    UBYTE *filePart;
    char *baseName;
    ULONG pathSize;
    ULONG partSize;
    ULONG position = 0;

    /* Returns Path with ':' or '/' */
 
    pathSize = strlen(path) * sizeof(UBYTE *) + 1;
 
    /* Get the file part which we need to ignore. */
    filePart = FilePart(path);
    partSize = strlen(filePart) * sizeof(UBYTE *) + 1;
 
    /* Allocate memory for the base name. */
    if((baseName = 
        AllocMem(
            strlen(path) * sizeof(UBYTE *) - strlen(filePart) + 1, 
            MEMF_ANY|MEMF_CLEAR
        )
    ) == NULL) return NULL;
 
    /* Allocate memory for path part. */
    if((buffer = AllocMem(pathSize, MEMF_ANY|MEMF_CLEAR)) == NULL) 
        return NULL;
 
    /* Strip pieces of the path and compose them. */
    do {
        position = SplitName(path, '/', buffer, position, pathSize);
 
        switch(position == -1 ||
            strlen(buffer) == 0 || 
            strncmp(buffer, filePart, partSize) == 0) 
            {
 
            case TRUE: break; /* We have found the file part. */
            default: /* Otherwise, copy the buffer into the base path. */
                sprintf(baseName + strlen(baseName), "%s/", buffer);
                break;
        }
 
    } while(position != -1);
 
    switch(strlen(baseName)) 
    {
        case 0: /* Attempt to build a drive path. */
            if(SplitName(path, ':', buffer, 0, pathSize) != -1)
                sprintf(baseName + strlen(baseName), "%s:", buffer);
            break;
        default: /* Otherwise, strip trailing slash. */
            //baseName[strlen(baseName)-1] = '\0';
            break;
    }
 
    return baseName;
}

/**************************************************************************************************/

void Make_Icon(char *image1, char *image2, char *outfile)
{	
	LONG offset = 0;
	LONG filebytes = 0;

	filebytes = GetFileSize(image1);
	Copy_File(image1, outfile, filebytes);
	offset = filebytes;
    filebytes = GetFileSize(image2);	
	Copy_File_Append(image2, outfile, offset, filebytes);
}

/**************************************************************************************************/

BOOL Save_Icon(struct DiskObject *oldicon, struct DiskObject *newicon, STRPTR name)
{	
	LONG type = 0; 
	LONG stack = 0;
	STRPTR tool = NULL;   
    UBYTE **ttypes = NULL;
	
	/* Get attributes of old icon */
    type = oldicon->do_Type;
	stack = oldicon->do_StackSize;
	tool = oldicon->do_DefaultTool;	
	ttypes = (UBYTE **)oldicon->do_ToolTypes;

	//Printf ("Save_Icon: %s\n", name);	

	//See: http://krashan.ppa.pl/articles/amigaicons/
	/* List of icon attributes */
    //dobj->do_Type
        //1 = disk or volume.
        //2 = drawer (folder).
        //3 = tool (executable).
        //4 = project (data file).
        //5 = trashcan.
        //6 = device.
        //7 = Kickstart ROM image.
        //8 = an appicon (placed on workbench by application).
    //dobj->do_ToolTypes
    //dobj->do_StackSize
    //dobj->do_Gadget
    //dobj->do_Gadget.Width 
    //dobj->do_Gadget.Height
    //dobj->do_DefaultTool	
    		
    switch(oldicon->do_Type)
    {
        case WBPROJECT:
		    newicon->do_Type = type;			
			newicon->do_DefaultTool = tool;       
            newicon->do_StackSize = stack;
            break;
        case WBTOOL: 
		    newicon->do_Type = type;           
			newicon->do_StackSize = stack;
            break;
    }	  
    newicon->do_ToolTypes = ttypes;

	if(oldicon->do_DrawerData)
	    newicon->do_DrawerData = oldicon->do_DrawerData;			 

    /* Write DiskObject to File */    
	if(PutDiskObject(name, newicon))
	{	
		//Printf ("PutDiskObject completed successfully.\n");    
	    return TRUE;
	} 
	//else
	    //Printf ("Save_Icon - PutDiskObject failed.\n");   
    
	return FALSE;	   
}

/**************************************************************************************************/

//void Exchange_Icon(struct DiskObject *dobj1, struct DiskObject *dobj2, char *destname)
void Exchange_Icon(struct DiskObject *dobj1, struct DiskObject *dobj2, char *destname, WORD deltasize, struct Window *win)
{	    
	LONG imSize = 0;	
    BOOL retval = FALSE;	

	tempname = "tempicon.info";
	tempfile = RemoveFileExtension(tempname);	

	/* Copy old icon attributes to new icon */    
	retval = Save_Icon(dobj1, dobj2, tempfile);	
    
    if(retval == TRUE)
	{
		/* Get temp icon file size */
	    imSize = GetFileSize(tempname);	
		//Printf ("File Size: %ld\n", imSize);		        
	    
	    /* Set dest icon file size */
		SetFilesize(destname,0); //SetFileSize(0) //BPTR lock?
		//Printf ("Set File Size Successful.\n");		

		Copy_Data(tempname,destname,imSize);
		//Printf ("Copy Data Successful.\n");	 
	    		
		DeleteFile(tempname);
	}
}

/**************************************************************************************************/

STRPTR GetDestinationFilename(char * asl_title, struct Window *win)
{
    STRPTR dstname = NULL;	
	if( DestFileReq == NULL )
    {
        DestFileReq = (struct FileRequester *)AllocAslRequestTags( ASL_FileRequest,
                                            ASLFR_Screen,          (win -> WScreen),                                            
											ASLFR_TitleText,       (IPTR)asl_title,                                            
                                            ASLFR_InitialLeftEdge, ((win -> LeftEdge) + (win -> BorderLeft) + 1UL),
                                            ASLFR_InitialTopEdge,  ((win -> TopEdge)  + (win -> BorderTop) + 1UL),
                                            ASLFR_DoPatterns,      TRUE,                                            
                                            TAG_DONE );
    }

    if( DestFileReq )
    {
        if( AslRequest( DestFileReq, NULL ) )
        {
            ULONG  buffsize;
            char *filebuffer = NULL;

            buffsize = strlen( (DestFileReq -> fr_Drawer) ) + strlen( (DestFileReq -> fr_File) ) + 10UL;

             if( filebuffer = (STRPTR)AllocVec( buffsize, MEMF_PUBLIC ) )
            {
                strcpy( filebuffer, (DestFileReq -> fr_Drawer) );
                AddPart( filebuffer, (DestFileReq -> fr_File), buffsize );
				dstname = filebuffer;                
            }
        }
    }
	else
	{
		/* User Cancelled */
		dstname = NULL;
	}
	return dstname;
}

/**************************************************************************************************/

STRPTR GetSourceFilename(char * asl_title, struct Window *win)
{	
	STRPTR source = NULL;
	if( SrcFileReq == NULL )
    {
        SrcFileReq = (struct FileRequester *)AllocAslRequestTags( ASL_FileRequest,
                                            ASLFR_Screen,          (win -> WScreen),                                            
											ASLFR_TitleText,       (IPTR)asl_title,                                            
                                            ASLFR_InitialLeftEdge, ((win -> LeftEdge) + (win -> BorderLeft) + 1UL),
                                            ASLFR_InitialTopEdge,  ((win -> TopEdge)  + (win -> BorderTop) + 1UL),
                                            ASLFR_DoPatterns,      TRUE,
                                            TAG_DONE );
    }

    if( SrcFileReq )
    {
        if( AslRequest( SrcFileReq, NULL ) )
        {
            ULONG  buffsize;
            char *filebuffer = NULL;

            buffsize = strlen( (SrcFileReq -> fr_Drawer) ) + strlen( (SrcFileReq -> fr_File) ) + 10UL;

             if( filebuffer = (STRPTR)AllocVec( buffsize, MEMF_PUBLIC ) )
            {
                strcpy( filebuffer, (SrcFileReq -> fr_Drawer) );
                AddPart( filebuffer, (SrcFileReq -> fr_File), buffsize );
				source = filebuffer;                
            }
        }
    }
	else
	{
		/* User Cancelled */
		source = NULL;
	}

    return source;
}

/**************************************************************************************************/

int getarguments(int argc, char **argv)
{
    int retvalue = 0;
	BOOL help = FALSE;
	BOOL open = FALSE;

	for (; argc > 1 && argv[1][0] == '-'; argc--, argv++) 
	{
        if (strcmp(argv[1],"--h") == 0) 
		{
			help = TRUE;            
            continue;
        }
        if (strcmp(argv[1],"--o") == 0) 
		{
			open = TRUE;            
            continue;
        }        
    }
	
	//if ((argc = 1) && (open == FALSE))
    //{
    	/* Print error message */		
		//Printf("Wrong number of arguments!\n");
		//retvalue = 0;
    //}
	if (help == TRUE)
	{
		/* Print help message */
        Printf("Usage: iconclone [--h] [--o] [srce icon] [dest icon]\n");
		Printf("Switch h: [iconclone --h] = show help\n");
		Printf("Switch o: [iconclone --o] = open gui\n");
		retvalue = 0;
	}

	if (argc == 0)    
    {    	
		//Arguments = 0 //argv[1] = --o		
		retvalue = 1; //no srce //use gui
		//Printf ("Arguments: %d\n", argc);			    
    }	
    if ((argc == 1) && (open == TRUE))    
    {    	
		//Arguments = 1 //argv[1] = --o		
		retvalue = 1; //no srce //use gui
		//Printf ("Arguments: %d\n", argc);	    
    }	
	if ((argc == 2) && (open == TRUE))    
    {    	
		//Arguments = 2 //argv[1] = --o
		retvalue = 2; //srce present, use gui
		//Printf ("Arguments: %d\n", argc);	    
    }	
	//if ((argc == 2) && (help == FALSE) && (open == FALSE))
	if ((argc == 3) && (open == FALSE))
    {		
		if (help == TRUE) 
		    retvalue = 0; //Arguments = 3 //argv[1] = --h
		else 			
		    retvalue = 3; //srce & dest present, use commandline
		//Printf ("Arguments: %d\n", argc);	    
    }
	if ((argc == 3) && (open == TRUE))
    {
		//Arguments = 3 //argv[1] = --o    	
		retvalue = 4; //srce & dest present, use gui
		//Printf ("Arguments: %d\n", argc);	    
    }

	return retvalue;
}

/**************************************************************************************************/

int main(int argc, char **argv)
{
	/* CLI Arguments & Switches */	
	BOOL open = TRUE; //Open GUI
	BOOL error = FALSE; //Error Handling
	int action = getarguments(argc, argv);
	//Printf ("Action Value: %d\n", action);

	/* Action Values */
	//action = 0;
	//action = 1;
	//action = 2;
	//action = 3;
	//action = 4;	

	/* CLI switches */
	//--o (open gui)
	//--h (show help)

	/* New commandline switch --t (int type 1-8) to change the destination icon type */
	/* The valiD return values start at 4 + 1,2,3,4 = 5,6,7,8 */
	//1 DISK - (disk)   /* WBDISK */
    //2 DRAWER - (directory)   /* WBDRAWER */
    //3 TOOL - (program)     /* WBTOOL */
    //4 PROJECT - (project)     /* WBPROJECT */	
	/* New function: Convert_Classic_Icon (change type) - add or remove drawerdata, defaultool as needed */
	/* See: old Save_Classic_Icon to add drawerdata, defaultool, etc. & copy data chunks to tempicon.info then rename */
	//void Change_Icon(char *infilename, int ictype);
    //void Change_Classic_Icon(char *infilename, int ictype);	

	/* Possible return values */	
	//"iconclone" returns 0  //Wrong # of arguments 
    //"iconclone --h" returns 0  //Wrong # of arguments //Show help
    //"iconclone --o" returns 1  //Open empty GUI
    //"iconclone more.info" returns 0  //Wrong # of arguments
    //"iconclone --h more.info" returns 0  //Wrong # of arguments //Show help
    //"iconclone --o more.info" returns 2  //Open GUI & display srce icon
	//"iconclone more.info editor.info" returns 3  //Don't open GUI just Exchange icons
    //"iconclone --h more.info editor.info" returns 0  //Show help   
    //"iconclone --o more.info editor.info" returns 4  //Open GUI & display srce icon & dest icon
	//Open from Workbench returns 1  //Arguments = 0  //Open empty GUI

	/* Valid return values */	
	//"iconclone --o" returns 1
	//"iconclone --o more.info" returns 2
	//"iconclone more.info editor.info" returns 3
	//"iconclone --o more.info editor.info" returns 4
	/* Note: remove quotes when using CLI */


	//char *infilename1 = ""; //srce file				
    //char *infilename2 = ""; //dest file


	//Use Switch & Case:
    //switch (action)
	//{
	//case 1:
        //Arguments = 2 //No srce present, use gui
		//open = TRUE; //argv[1] = --o       														
		//break;
	//case 2:

    if (action == 0) //No srce present, Don't use gui
    {
    	open = FALSE;
		//No srce icon //Don't open gui
    }
	if (action == 1) //Arguments = 1 or 2 //No srce present, use gui
    {
    	open = TRUE; //argv[1] = --o
		//No srce icon //Open gui
    }
	if (action == 2) //Arguments = 3 //srce present, use gui
    {
		open = TRUE; //argv[1] = --o
    	srcename = argv[2]; //srce icon
		//infilename1 = argv[2]; //srce icon
		//if(srcename) Printf ("Srce Name: %s\n", srcename);
		//See: PicsViewer x86_64 for IsIcon function				
		/* Check for .info file extension */ //BOOL IsIcon(srcename)
		//if(IsIcon != TRUE)
		//{
			//open = FALSE; 
			//action == 0;
		//} 
    }
	if (action == 3) //Arguments = 3 //srce & dest present, use commandline
    {		
    	open = FALSE;
		srcename = argv[1]; //srce icon				
    	destname = argv[2]; //dest icon		
		//infilename1 = argv[1]; //srce icon				
    	//infilename2 = argv[2]; //dest icon		
		//if(srcename) Printf ("Srce Name: %s\n", srcename);
		/* Check for .info file extension */ //BOOL IsIcon(srcename)		
		//if(destname) Printf ("Dest Name: %s\n", destname);
		/* Check for .info file extensions */ //BOOL IsIcon(destname)
		//if(IsIcon != TRUE)
		//{			 
			//action == 0;
		//}		        	    
    }
	if (action == 4) //Arguments = 4 //srce & dest present, use gui
    {
		open = TRUE; //argv[1] = --o
    	srcename = argv[2]; //srce icon				
    	destname = argv[3]; //dest icon
		//infilename1 = argv[2]; //srce icon				
    	//infilename2 = argv[3]; //dest icon		
		//if(srcename) Printf ("Srce Name: %s\n", srcename);
		/* Check for .info file extension */ //BOOL IsIcon(srcename)		
		//if(destname) Printf ("Srce Name: %s\n", destname);
		/* Check for .info file extensions */ //BOOL IsIcon(destname)
		//if(IsIcon != TRUE)
		//{
			//open = FALSE; 
			//action == 0;
		//}	    
    }

    BPTR fh;
	if(srcename)
	{
	    /* Check if icon file exists */
        fh = Open(srcename, MODE_OLDFILE);
        if (!fh)
		{
			/* Error */
			error = TRUE;			
		}
		Close(fh);
	}
	if(destname)
	{
	    /* Check if icon file exists */
        fh = Open(destname, MODE_OLDFILE);
        if (!fh)
		{
			/* Error */
			error = TRUE;			
		}
		Close(fh);
	}    

	
	/* User Interface based on several GadTools samples */
    struct Gadget *gad;		/* The Actual Gadget */
    struct NewGadget ng;	/* Various Information about new Gadgets */
    struct Screen *scr;
    struct Window *win;
    struct MsgPort *app_port;
    struct AppWindow *app_window;
    struct AppMessage   *amsg;
    struct IntuiMessage *imsg;
    struct DrawInfo *dri;
    BOOL reverse = FALSE;
    UBYTE namebuffer[300];
    ULONG TOPIMAGE;
    ULONG LEFTIMAGE_X;
    ULONG RIGHTIMAGE_X;
    ULONG winmask;
    ULONG msgmask;
    ULONG sigs;
    BOOL cont;
    char *location;
    long winw,winh;	


    if (scr = LockPubScreen (NULL))
	{
	    glist = NULL;
	    gad = CreateContext (&glist);	/* Here we initialize the Gadget List */

        if ((dri = GetScreenDrawInfo(scr)))
        {
    	    topoffset = dri->dri_Font->tf_YSize + scr->WBorTop - 10 + 5;
	    }

	    /* We set these values for all Gadgets to use */
	    ng.ng_VisualInfo = GetVisualInfo (scr,TAG_END);
	    ng.ng_TextAttr   = scr->Font;
	    ng.ng_Flags      = 0;

	    leftoffset = scr->WBorLeft + 4;

	    /* The following block (including CreateGadget) is repeated for each new gadget. 
        Only the values ​​that change are adapted.
	    The previous gadget is passed to CreateGadget so that the 
        Concatenation can be established.
	    It is not necessary to check every time whether a result has been achieved.    
        If the previous gadget was NULL, all further gadgets will also be NULL, so  
        you only have to check once at the end. */

	    //struct NewGadget
        //{
        //WORD ng_LeftEdge, ng_TopEdge;  /* gadget position */
        //WORD ng_Width, ng_Height;      /* gadget size */
        //UBYTE *ng_GadgetText;          /* gadget label */
        //struct TextAttr *ng_TextAttr;  /* desired font for gadget label */
        //UWORD ng_GadgetID;             /* gadget ID */
        //ULONG ng_Flags;                /* see below */
        //APTR ng_VisualInfo;            /* Set to retval of GetVisualInfo() */
        //APTR ng_UserData;              /* gadget UserData */
        //};

	    /* Note: Use struct TextAttr *ng_TextAttr to specify the gadget font */
	    /* Default is to use the screen font & size which may be too large */	

	    ng.ng_LeftEdge   = scr->WBorLeft + 4 + 8 * scr->RastPort.TxWidth;
	    ng.ng_TopEdge    = scr->WBorTop + scr->RastPort.TxHeight + 5;
	    ng.ng_Width      = 20 * scr->RastPort.TxWidth + 80; //20
	    ng.ng_Height     = scr->RastPort.TxHeight + 6;

	    if(ng.ng_Height > 19)
	    {
	        buttonoffset     = ng.ng_Height - 19;
	        windowsize       = buttonoffset * 4;
	        deltasize        = windowsize / 2;
	    }
	    else
	    {
		    buttonoffset     = 0;
	        windowsize       = 0;
	        deltasize        = 0;
	    }
    

	    /* Center buttons inside outer groupbox */
	    //Outer groupbox = 228 + windowsize, 105 + windowsize

	    //ng.ng_LeftEdge   = scr->WBorLeft + 4 + 10 * scr->RastPort.TxWidth;
	    ng.ng_TopEdge   += ng.ng_Height + 4;
	    ng.ng_Width      = 20 * scr->RastPort.TxWidth + 60;	

	    //ng.ng_LeftEdge = 20 * scr->RastPort.TxWidth + scr->WBorLeft + 4;
	    ng.ng_LeftEdge = leftoffset + 105 + deltasize;
	    //ng.ng_TopEdge   += ng.ng_Height + 4;
	    ng.ng_Width      = 20;
	    ng.ng_GadgetText = "?";
	    //ng.ng_TextAttr = &topaz11;
	    //ng.ng_TextAttr = &arial20;
	    ng.ng_GadgetID   = GID_REQ1;
	    butgad1 = gad = CreateGadget (BUTTON_KIND,gad,&ng,TAG_END);

	    //ng.ng_LeftEdge = 20 * scr->RastPort.TxWidth + scr->WBorLeft + 0;
	    ng.ng_LeftEdge = leftoffset + 100 + deltasize;
	    ng.ng_TopEdge   += ng.ng_Height + 4;
	    ng.ng_Width      = 30;
	    ng.ng_GadgetText = ">>";
	    //ng.ng_TextAttr = &topaz11;
	    //ng.ng_TextAttr = &arial20;
	    ng.ng_GadgetID   = GID_EXCH;
	    butgad2 = gad = CreateGadget (BUTTON_KIND,gad,&ng,TAG_END);

	    ng.ng_LeftEdge   = scr->WBorLeft + 74;
	    ng.ng_TopEdge   += ng.ng_Height + 4;
	    ng.ng_Width      = 20 * scr->RastPort.TxWidth + 60;	

	    //ng.ng_LeftEdge = 20 * scr->RastPort.TxWidth + scr->WBorLeft + 4;
	    ng.ng_LeftEdge = leftoffset + 105 + deltasize;
	    //ng.ng_TopEdge   += ng.ng_Height + 4;	
	    ng.ng_Width      = 20;
	    ng.ng_GadgetText = "?";
	    //ng.ng_TextAttr = &topaz11;
	    //ng.ng_TextAttr = &arial20;
	    ng.ng_GadgetID   = GID_REQ2;
	    butgad3 = gad = CreateGadget (BUTTON_KIND,gad,&ng,TAG_END);

	    ng.ng_LeftEdge   = scr->WBorLeft + 4;
	    ng.ng_TopEdge   += ng.ng_Height + 4;
	    ng.ng_Width      = 15 * scr->RastPort.TxWidth + 8;	

	    ng.ng_LeftEdge  += ng.ng_Width + 64;
	
	    winw = leftoffset + 13 + 228 + windowsize; 
	    winh = topoffset + 23 + 105 + windowsize;

		//GadTools Library should use FrontPen/BackPen for Buttons, Checkboxes, TextBoxes, Menus, etc.
		//GTTX_FrontPen (UBYTE) - The pen to use when rendering the text. (V39)
	    //GTTX_BackPen (UBYTE) - The pen to use when rendering the background
		//Use SetAPen & SetBPen to set pen colors for Foreground & Background




	    /* Use commandline */
	    //if ((infilename1) && (infilename2) && (open == FALSE))	    
		//if ((infilename1) && (infilename2) && (open == FALSE))
		if (open == FALSE)
        {
			if (error == FALSE)
			{
                if (action == 0) //Arguments < 3
                {    	        
		            //No srce icon //Don't open gui
				    //Printf ("Wrong number of arguments. Don't open GUI.\n");
                }
			    else if (action == 3) //Arguments = 3
			    {
				    //srce & dest present, use commandline
		            //if(srcename) Printf ("Commandline - Srce Name: %s\n", srcename);		        
		            /* Srce Icon File has .info Extension */
		            srcefile = RemoveFileExtension(srcename);
					//Remove .info File Extension
					//srcefile = srcename;
					//srcefile[strlen(srcename) - 5] = 0;
				    //if(srcefile) Printf ("Commandline - Srce File: %s\n", srcefile);		
		            srcedobj = GetDiskObjectNew(srcefile);

				    //if(destname) Printf ("Commandline - Dest Name: %s\n", destname);
		            /* Dest Icon File has .info Extension */
		            destfile = RemoveFileExtension(destname);
					//Remove .info File Extension
					//destfile = destname;
					//destfile[strlen(destname) - 5] = 0;
				    //if(destfile) Printf ("Commandline - Dest File: %s\n", destfile);
		            destdobj = GetDiskObjectNew(destfile);

		            /* Perform Icon Exchange */
				    if(srcename != NULL && destname != NULL)
				    {
                        Exchange_Icon(destdobj, srcedobj, destname, deltasize, win);	
			            //Printf ("Icon Exchange Completed Successfully.\n");
				    }	
			    }
			}
			else
			{
		        Printf("Loading Icon Failed. Bad Filename.\n");
				/* Set Error */
				error = FALSE;
				//Goto exit	
			}			
	    }

	    
	    /* Open the GUI */
	    if (open == TRUE) //if(open)
		//if (open == TRUE && error == FALSE)
	    {
	        if (gad)	/* Check whether all gadgets have been created */
	        {
		        //Create the AppMsg_Port
		        app_port = CreateMsgPort();		

		        /* When opening the window, the IDCMP flags for Gadtools gadgets must be specified. */
                /* The constants <typ>IDCMP are used for this. */

				//WA_BlockPen,(scr->BlockPen)
				//WA_DetailPen,(scr->DetailPen)
				//Screen SA_DetailPen, SA_BlockPen, SA_Pens (3D Look)
                //SetRGB4(color) //change screen pen color
                //Window DetailPen, BlockPen

				//WA_DetailPen and WA_BlockPen
                //WA_DetailPen specifies the pen number for the rendering of window details like gadgets or text in the title bar. 
				//WA_BlockPen specifies the pen number for window block fills, like the title bar. These pens are also used for 
				//rendering menus. Equivalent to NewWindow.DetailPen and NewWindow.BlockPen.
                //The specific color associated with each pen number depends on the screen. Specifying -1 for these values sets 
				//the window's detail and block pen the same as the screen's detail and block pen.
                //Detail pen and block pen have largely been replaced starting with V36 by the pen array in the DrawInfo structure. See the section on "DrawInfo and the 3D Look" in Intuition Screens for more information.
		
		        if (win = OpenWindowTags (NULL,
				WA_Width,winw,
				WA_Height,winh,
				WA_Left,(scr->Width - winw) / 2,	
				WA_Top,(scr->Height - winh) / 2,
				WA_PubScreen,scr,
				WA_Title,"Icon Clone v1.09",				
				WA_BlockPen,(scr->BlockPen),
				WA_DetailPen,(scr->DetailPen),
				WA_Flags,WFLG_CLOSEGADGET | WFLG_DRAGBAR | WFLG_DEPTHGADGET | WFLG_ACTIVATE,
				WA_IDCMP,IDCMP_CLOSEWINDOW | IDCMP_VANILLAKEY | IDCMP_REFRESHWINDOW | BUTTONIDCMP | CYCLEIDCMP | STRINGIDCMP,
				WA_Gadgets,glist,
				TAG_END))
			    {
			        GT_RefreshWindow (win,NULL);	/* Redraw all Gadtools gadgets. */
											/* This has to be done once at the beginning. */

			        UnlockPubScreen (NULL,scr);		/* the window blocks the screen anyway, */
			        scr = NULL;						/* therefore the lock is no longer required here. */

                    /* DRAW THREE BEVEL BOXES */
	                //The outer Bevel Box is the group box.
	                //The two inner Bevel Boxes are raised panels.

					//GTBB_Recessed (BOOL)
                    //A beveled box may either appear to be raised to signify an area of the window that is selectable 
					//or recessed to signify an area of the window in which clicking will have no effect. Set this boolean tag 
					//to TRUE to get a recessed box. Omit this tag entirely to get a raised box.
                    //DrawBevelBox() is a rendering operation, not a gadget. This means that the program must refresh any beveled 
					//boxes rendered through this function if the window gets damaged.	        

			        /* First Raised Panel */ //26,151,15-3 //leftoffset + 13,leftoffset + 138,leftoffset + 2		
                    DrawBevelBox(win->RPort, leftoffset + 13 + deltasize, 25 + topoffset + deltasize, 80, 80,
                         GT_VisualInfo, (IPTR) ng.ng_VisualInfo, TAG_DONE);
	                /* Second Raised Panel */
                    DrawBevelBox(win->RPort, leftoffset + 138 + deltasize, 25 + topoffset + deltasize, 80, 80,
                         GT_VisualInfo, (IPTR) ng.ng_VisualInfo, TAG_DONE);    				 
	                /* Outer Group Box */
                    DrawBevelBox(win->RPort, leftoffset + 2, 12 + topoffset, 228 + windowsize, 105 + windowsize,
                         GTBB_FrameType, BBFT_RIDGE, GTBB_Recessed, TRUE,
                         GT_VisualInfo, (IPTR) ng.ng_VisualInfo, TAG_DONE);

                    /* Offsets used for Rectangle Coords */
			        TOPIMAGE = 25 + topoffset + deltasize;
			        LEFTIMAGE_X = leftoffset + 13 + deltasize;
			        RIGHTIMAGE_X = leftoffset + 138 + deltasize;		

            

			        if (error == FALSE)
					{
					    /* Open GUI & Display Icons */
	                    //if ((strlen(infilename1) > 0) && (open == TRUE)) //action = 2,4
			            //if ((strlen(srcename) > 0) && (open == TRUE))
					    if ((action == 2) || (action == 4))
                        {								
                            /* Srce Icon */																	    
		                    reverse = FALSE;
		                    location = "left";
		                    /* Srce Icon File has .info Extension */
		                    //srcefile = RemoveFileExtension(infilename1);
				            if(srcename) srcefile = RemoveFileExtension(srcename);
							//Remove .info File Extension
					        //srcefile = srcename;
					        //srcefile[strlen(srcename) - 5] = 0;
				            //if(srcefile) Printf ("Srce File: %s\n", srcefile);		
		                    srcedobj = GetDiskObjectNew(srcefile);

		                    /* Display Srce Icon Image */		
		                    //Display_Icon(srcename, location, deltasize, win);
						    Display_Icon(srcename, srcedobj, location, deltasize, win);				    
                        }	                
	                    //if ((strlen(infilename2) > 0) && (open == TRUE)) //action = 4
			            //if ((strlen(destname) > 0) && (open == TRUE))
					    if (action == 4)
                        {
		                    /* Dest Icon */
		                    reverse = FALSE;
		                    location = "right";
		                    /* Dest Icon File has .info Extension */
		                    //destfile = RemoveFileExtension(infilename2);
				            if(destname) destfile = RemoveFileExtension(destname);
							//Remove .info File Extension
					        //destfile = destname;
					        //destfile[strlen(destname) - 5] = 0;
				            //if(destfile) Printf ("Dest File: %s\n", destfile);
		                    destdobj = GetDiskObjectNew(destfile);

		                    /* Display Dest Icon Image */							
                            //Display_Icon(destname, location, deltasize, win);
						    Display_Icon(srcename, destdobj, location, deltasize, win);				    
                        }
					}
					else
					{
						/* Show Output Message */
		                //STRPTR outputmsg = "Loading Icon Failed. Bad Filename.\n";
		                //OutputMessage(outputmsg, win);
						Printf("Loading Icon Failed. Bad Filename.\n");
						/* Set Error */
						error = FALSE;
						//Goto exit
					}
			
	        

                    /* Setup an AppWindow using MsgPort app_port */
			        if(!(app_window = AddAppWindow(0, 0, win, app_port, NULL)))
                    {
                        /* Can't Add Appwindow! */
				        Printf ("Can't Add Appwindow!\n");
			        }
                    else
			        {				
				        /* AppWindow added... Drag Icons into AppWindow */
				        //Printf("AppWindow added... Drag Icons into AppWindow\n");               
				        /* Setup our signals */
			            winmask = 1L << win->UserPort->mp_SigBit;
			            msgmask = 1L << app_port->mp_SigBit;			


			            /* This is followed by a normal input query for a window. */
                        /* Gadtools functions GT_GetIMsg & GT_ReplyIMsg are used instead of GetMsg & ReplyMsg */
			            cont = TRUE;
			            do	
			            {
				            /* Wait for IDCMP & AppMessages */				
				            sigs = Wait(winmask | msgmask);
                                               				
				            if (sigs & msgmask) /* Got an AppMessage */
				            { 
					            /* User dropped an Icon! Get the message from the app_port */					
				                while ( (app_port) && (amsg = (struct AppMessage *) GetMsg(app_port)) )
                                {											        
                                    if (amsg->am_NumArgs >= 1)
							        //if(amsg->am_NumArgs>0L) // could it be less than 0 ? 
                                    {
                                        NameFromLock(amsg->am_ArgList->wa_Lock, namebuffer, 299);
                                        AddPart(namebuffer, amsg->am_ArgList->wa_Name, 299);
                                        filename = namebuffer;
										//Printf ("Appwindow Message FileName: %s\n", filename);

										/* Length of filename */
										//int x = strlen(filename);

										/* If filename contains '.' dot then that may indicate 2 causes */
										/* It will be a file with def icon or an icon with bad filename */
										/* When we drag-n-drop there should be no .info file extension! */
										char *ptr;
										for (ptr = filename; *ptr!= '\0'; ptr++)
										{
											if (*ptr == '.')
											{
												/* Check .info file extension */
												char *ext = get_ext(filename); 								
								                if(strnicmp((const char *)ext, "info", 4)==0)
								                {                                                    
													//error = TRUE;
													//Remove .info File Extension
										            filename[strlen(filename) - 5] = 0; 
												}
												else
												{
												    //else it is ok to copy the def icon
												    srcedobj = GetDiskObjectNew(filename);
												    PutDiskObject(filename, srcedobj);
												} 																								
											}
											else
											{
										            //else it is ok to copy the def icon
											    srcedobj = GetDiskObjectNew(filename);
											    PutDiskObject(filename, srcedobj);
											}
										}

										//RemoveFileExtension(filename);
										//if(!strncmp(filename + x - 5, ".info", 5)) filename[x - 5] = 0;


										//s = strlen(args.icon);
                                        //if(!strncmp(args.icon+s-5, ".info",5))
                                        //args.icon[s-5] = 0;									

								        /* Find trailing slash for drawers*/
								        if(filename[strlen(filename)-1] == '/')
								        {
								            //Printf ("Appwindow Message - Strip trailing slash: %s\n", filename);
											/* Strip trailing slash */
                                            filename[strlen(filename)-1] = '\0'; //= 0; //'\0'
											//filename[x-1] = '\0';									        											
								        }
										/* Find trailing ':' for volumes */
								        if(filename[strlen(filename)-1] == ':')
								        {
											//Printf ("Appwindow Message - Found trailing volID: %s\n", filename);
											/* New disk filename */
											tempname = filename;												
											int x = strlen(filename);
											sprintf(tempname + x, "%s", "disk.info");
											filename = RemoveFileExtension(tempname); 
											//RemoveFileExtension(tempname) = tempname[x - 5] = 0;

											//RemoveFileExtension(filename);
										    //if(!strncmp(filename + x - 5, ".info", 5)) filename[x - 5] = 0;

										
										    //if (argc == 0)
											//{
												/* Opened from Workbench */

												/* New disk filename */
												//tempname = filename; 												
												//int x = strlen(filename);												
												//sprintf(tempname + x, "%s", "Disk.info");
												//sprintf(tempname + x, "%s", "disk.info");
												//filename = RemoveFileExtension(tempname);

												//BPTR file; /* Check disk filename */
												//if(file = Open(tempname, MODE_OLDFILE))
												//{
													//Close(file);													
													//filename = RemoveFileExtension(tempname);
												//}
												//else
												//{
													/* Show Output Message for Icon Load Error */
		                                            //STRPTR outputmsg = "Error! Can't load volume icon.";
		                                            //OutputMessage(outputmsg, win);
												//}											    												
											//}
											//else
											//{																								
												/* Opened from commandline */
                                                
                                                /* New disk filename */
												//tempname = filename;												
												//int x = strlen(filename);												
												//sprintf(tempname + x, "%s", "Disk.info");
												//sprintf(tempname + x, "%s", "disk.info");
												//filename = RemoveFileExtension(tempname);

												/* Treat this as Copy_Icon for Disk.info */
											    //Workbench:Prefs/Env-Archive/Sys/def_ram.info to ram:disk.info
                                                //you may have to reboot or get Workbench to refresh or redraw icons
                                                //copy the icon you want to ram:disk.info then from CLI 'diskchange ram:'																																																
											//}											
								        }
                                        												
                                        /* Add .info to name */
										int x = strlen(filename);	        
                                        sprintf(filename + x, "%s", ".info");
										
                                        /* Check if filename exists */
										//fh = Open(filename, MODE_OLDFILE);
										//if (!fh)
										//{
											/* Error */
			                                //error = TRUE;
										//}
										//Close(fh);	
                                                                        
                                    }
							        // No we've got the filename of the icon that was dropped we can do stuff with it.
                                    // So first lets determine if the user dropped the icon in the source or destination
                                    // boxes of window.
                            
                                    if (inRectangle(LEFTIMAGE_X, TOPIMAGE, MAX_IMAGE_WIDTH, MAX_IMAGE_HEIGHT, amsg->am_MouseX, amsg->am_MouseY))
                                    {
                                        /* Icon dropped in Srce box */
								        //Printf ("Icon dropped in Srce box\n");															    
								        location = "left";

								        /* Erase Icon Image Left */ 
								        //EraseRect(win->RPort, leftoffset + 13 + 1 + deltasize, win->BorderTop + topoffset + 1 + deltasize,
                                        //win->Width - 145 - win->BorderRight - deltasize,
                                        //win->Height - 20 - win->BorderBottom - deltasize);								

								        //reverse = FALSE;

                                        /* Assign source icon */
								        //srcename = RemoveFileExtension(filename);
										//int x = strlen(filename);
										srcename = filename;
										srcename[strlen(filename) - 5] = 0; //Remove .info File Extension										
										//Printf ("inRectangle Message SrceFile: %s\n", srcename);							
								        srcedobj = GetDiskObjectNew(srcename);																 

								        /* Check if icon file exists */
                                        BPTR fh = Open(filename, MODE_OLDFILE);
                                        if (!fh) /*if file doen not exist */
								        {   
									        /* Write DiskObject to File */
											//Printf ("Write DiskObject to File: %s\n", srcename);
									        PutDiskObject(srcename, srcedobj);
								        }
								        Close(fh);

										/* Add .info to name */
								        sprintf(srcename + strlen(srcename), "%s", ".info");
										//Printf ("inRectangle Message FilePath: %s\n", srcename);							                                 
                                    }
                                    else if (inRectangle(RIGHTIMAGE_X, TOPIMAGE, MAX_IMAGE_WIDTH, MAX_IMAGE_HEIGHT, amsg->am_MouseX, amsg->am_MouseY))
                                    {
                                        /* Icon dropped in Dest box */
								        //Printf ("Icon dropped in Dest box\n");							    
							            location = "right";

								        /* Erase Icon Image Right */
								        //EraseRect(win->RPort, leftoffset + 138 + 1 + deltasize, win->BorderTop + topoffset + 1 + deltasize,
                                        //win->Width - 20 - win->BorderRight - deltasize,
                                        //win->Height - 20 - win->BorderBottom - deltasize);								

								        //reverse = FALSE;

										// Try to get destination icon
	                                    //if	(!(dest = GetNewDiskObject( (APTR)destname )))
		                                //dest = GetDefNewDiskObject( data->icon_type );

                                        /* Assign destination icon */
								        //destname = RemoveFileExtension(filename);
										//int x = strlen(filename);
										//filename[x - 5] = 0; //Remove .info File Extension
										//destname = filename;
										//Printf ("inRectangle Message SrceFile: %s\n", destname);								
								        //destdobj = GetDiskObjectNew(destname);

										/* Assign destination icon */								        
										destname = filename;
										destname[strlen(filename) - 5] = 0; //Remove .info File Extension
								        destdobj = GetDiskObjectNew(destname);
								
								        /* Check if icon file exists */
                                        BPTR fh = Open(filename, MODE_OLDFILE);
                                        if (!fh) /*if file doen not exist */
								        {
									        /* Write DiskObject to File */
											//Printf ("Write DiskObject to File: %s\n", destname);
									        PutDiskObject(destname, destdobj);
								        }
								        Close(fh);

										/* Add .info to name */
								        sprintf(destname + strlen(destname), "%s", ".info");
										//Printf ("inRectangle Message FilePath: %s\n", srcename);                        
                                    }
                                    if (filename)									
                                    {
										if (error == FALSE)
										{
										    if(stricmp(location, "left")==0)
		                                    {
			                                    //srceicon
											    Display_Icon(filename, srcedobj, location, deltasize, win);
                                                //Display_Rectangle("red", "left", deltasize, win);
		                                    }
		                                    else if(stricmp(location, "right")==0)
		                                    {
			                                    //desticon
											    Display_Icon(filename, destdobj, location, deltasize, win);
                                                //Display_Rectangle("red", "right", deltasize, win);
		                                    }
										}																	
                                        else
										{
                                            if (argc == 0)
											{
												/* Show Output Message */
		                                        STRPTR outputmsg = "Loading Icon Failed. Bad Filename.\n";
		                                        OutputMessage(outputmsg, win);												
												/* Set Error */
												error = FALSE;
												//Goto exit
											}
											else
											{																									
												Printf("Loading Icon Failed. Bad Filename.\n");
												/* Set Error */
												error = FALSE;
												//Goto exit
											}
										}
                                    }                        

                                    ReplyMsg ((struct Message *) amsg);						
                                    ActivateWindow(win);                        
                                } /* while ((amsg = (struct AppMessage *) GetMsg(app_port))) */
				            } //Sigs & msgmask


				            // User clicked a gadget or something				      
                            if (sigs & winmask) /* Got an IDCMP message */
                            {
				                while (imsg = GT_GetIMsg (win->UserPort))
				                {
					                switch (imsg->Class)
						            {
					                case IDCMP_GADGETUP:
						                gad = (struct Gadget *) imsg->IAddress;
						                switch (gad->GadgetID)
						                {												
						                case GID_REQ1:
							                /* Get Source Filename */
							                srcename = GetSourceFilename("Source File...", win);
											//Printf ("Get Source Filename: %s\n", srcename);
							                if(srcename != NULL && strlen(srcename) > 0)							
							                {
							                    /* Erase Icon Image Left */ 	
	                                            //EraseRect(win->RPort, leftoffset + 13 + 1 + deltasize, win->BorderTop + topoffset + 1 + deltasize,
                                                //win->Width - 145 - win->BorderRight - deltasize,
                                                //win->Height - 20 - win->BorderBottom - deltasize);

								                //reverse = FALSE;

								                /* Check Icon Name */
								                char *ext = get_ext(srcename); 								
								                if(strnicmp((const char *)ext, "info", 4)!=0)
								                { 	
									                //Printf ("Source File has no .info Ext\n");	
									                srcedobj = GetDiskObjectNew(srcename);							
								                    sprintf(srcename + strlen(srcename), "%s", ".info");
									                srcefile = RemoveFileExtension(srcename);											
								                }
								                else
								                {   
									                /* Source Icon File has .info Extension */
									                srcefile = RemoveFileExtension(srcename);
								                    //Printf ("Source File: '%s'\n", srcefile);
									                srcedobj = GetDiskObjectNew(srcefile);
								                }                                

                                                /* Check if icon file exists */
                                                BPTR fh = Open(srcename, MODE_OLDFILE);
                                                if (!fh) /*if file doen not exist */
								                {
									                /* Write DiskObject to File */
													//Printf ("Write DiskObject to File: %s\n", srcefile);
									                PutDiskObject(srcefile, srcedobj);
								                }
								                Close(fh);
								
								                /* Display Icon Image Left */
							                    //retval = Display_Icon(srcename, "left", deltasize, win);
												//retval = Display_Icon(srcename, srcedobj, "left", deltasize, win);
								                //if(retval == FALSE) srcename = NULL;
												Display_Rectangle("red", "left", deltasize, win);								
							                }
							                else
							                {
								                /* Erase Icon Image Left */ 	
	                                            EraseRect(win->RPort, leftoffset + 13 + 1 + deltasize, win->BorderTop + topoffset + 1 + deltasize,
                                                win->Width - 145 - win->BorderRight - deltasize,
                                                win->Height - 20 - win->BorderBottom - deltasize);
								                //Printf ("\nNothing Selected!\n");
                                                srcename = NULL;
							                }
							                break;
						                case GID_EXCH:
							                if(srcename != NULL && destname != NULL)														
							                {	 
							                    //if(reverse == TRUE)
								                //{
									                /* Single-Click to Reverse Icon Exchange */
									                //srcedobj = destdobj; /* switch disk objects, source is old destination */
									                //destfile = RemoveFileExtension(destname);
								                    //destdobj = GetDiskObject(destfile); //current destination icon
                                                    //Exchange_Icon(destdobj, srcedobj, destname, deltasize, win);
									                /* reverse was true. should remain true till next drop */								
								                //}
								                //else
								                //{
													//reverse = TRUE; /* reverse was false */

													/* Erase Window RasterPort "right" */	
	                                                //EraseRect(win->RPort, leftoffset + 138 + 1 + deltasize, win->BorderTop + topoffset + 1 + deltasize,
                                                              //win->Width - 20 - win->BorderRight - deltasize,
                                                              //win->Height - 20 - win->BorderBottom - deltasize);


													/* Exchange icon images */
								                    Exchange_Icon(destdobj, srcedobj, destname, deltasize, win);

													/* Rebuild destination diskobject */
													destfile = RemoveFileExtension(destname);
													destdobj = GetDiskObjectNew(destfile);

                                                    /* If GUI open display dest image */
													if (open == TRUE) Display_Icon(destname, destdobj, "right", deltasize, win);
													
								                //}															    								
							                }														
							                break;
						                case GID_REQ2:						    
						                    /* Get Destination Filename */
							                destname = GetDestinationFilename("Destination...", win);
											//Printf ("Get Destination Filename: %s\n", destname);
							                if(destname != NULL && strlen(destname) > 0)							
							                {
                                                /* Erase Icon Image Right */	
	                                            //EraseRect(win->RPort, leftoffset + 138 + 1 + deltasize, win->BorderTop + topoffset + 1 + deltasize,
                                                //win->Width - 20 - win->BorderRight - deltasize,
                                                //win->Height - 20 - win->BorderBottom - deltasize);

								                //reverse = FALSE;
												
                                                /* Check Icon Name */
								                char *ext = get_ext(destname);							
								                if(strnicmp((const char *)ext, "info", 4)!=0)
								                { 									
								                //Printf ("Destination File has no .info Ext\n");	
									            destdobj = GetDiskObjectNew(destname);							
								                sprintf(destname + strlen(destname), "%s", ".info");
									            destfile = RemoveFileExtension(destname);											
								                }
								                else
								                {   
									                /* Destination Icon File has .info Extension */
									                destfile = RemoveFileExtension(destname);
								                    //Printf ("Destination File: '%s'\n", destfile);
									                destdobj = GetDiskObjectNew(destfile);
								                }								

								                /* Check if icon file exists */
                                                BPTR fh = Open(destname, MODE_OLDFILE);
                                                if (!fh) /*if file doen not exist */
								                {
									                /* Write DiskObject to File */
													//Printf ("Write DiskObject to File: %s\n", destfile);
									                PutDiskObject(destfile, destdobj);
								                }
								                Close(fh);								

								                //Display Icon Image Right						
							                    //retval = Display_Icon(destname, "right", deltasize, win);
												//retval = Display_Icon(destname, destdobj, "right", deltasize, win);
								                //if(retval == FALSE) destname = NULL;
												Display_Rectangle("red", "right", deltasize, win);
							                }
							                else
							                {
								                /* Erase Icon Image Right */	
	                                            EraseRect(win->RPort, leftoffset + 138 + 1 + deltasize, win->BorderTop + topoffset + 1 + deltasize,
                                                win->Width - 20 - win->BorderRight - deltasize,
                                                win->Height - 20 - win->BorderBottom - deltasize);
								                //Printf ("\nNothing Selected!\n");
                                                destname = NULL;
							                }																											
							                break;						
						                }
						            break;
					                case IDCMP_VANILLAKEY:
						                if (imsg->Code == 0x1b) /* Esc */
							                cont = FALSE;
						                break;
					                case IDCMP_CLOSEWINDOW:
						                cont = FALSE;
						                break;
					                case IDCMP_REFRESHWINDOW:
						                GT_BeginRefresh(win);
						                GT_EndRefresh(win, TRUE);
						                break;
						            }
					                GT_ReplyIMsg (imsg);
				                }
				            } //Sigs & winmask
				
			            } //Do-While Loop
			            while (cont); /* done */
			            //while ((cont) && (open)); /* done */
			            RemoveAppWindow(app_window);
			        } //AppWindow added

			    CloseWindow (win);
			    } //if (win)
			    /* Make sure there are no more outstanding messages */
                while(amsg = (struct AppMessage *)GetMsg(app_port))
                  ReplyMsg((struct Message *)amsg);                  
                DeleteMsgPort(app_port);			
		    } //if (gad)
        } //Open the GUI
    
	FreeGadgets (glist);

	FreeVisualInfo (ng.ng_VisualInfo);	

	if (scr) UnlockPubScreen (NULL,scr);
	}
	FreeDiskObject(srcedobj);
	FreeDiskObject(destdobj);

return (0);
}